/*

************** PSO 8.2: An Adaptive Particle Swarm Optimizer  ******************************

 Maurice.Clerc@WriteMe.com 		2001-10-06      (ANSI C) 

What's new since?
(done) save data and post-mortem visualisation (cf Visu. Needs Leonardo free C compiler. See below)
(done) adaptive neighbourhood size for each particle 
(done) option "local queen"
(done) option "guided random generation"
(partly done) new estimation of the number of evaluations still needed 
(partly done) new metaphore similar to update_vel2, but for all kind of problems

Some future options
(in progress ) adaptive neighbourhood (each particule can choose its neighbours depending how much it trusts them)
(in progress) multicriteria optimization
(in progress) TSP

WARNING. There are probably still some bugs for energies (potential and kinetic)

This version does not _need_ any parameters at all, except, of course, the ones
defining the problem (file problems.txt):
- function
- dimension
- xmin,xmax, granularity (for the hyperparallepipedic search space)
- target
- admissible error
- granularity
- if all components of the solution has to be different (useful for some combinatorial problems)	

Nevertheless, for research purpose, you can modify the standard behaviour (see models.txt file).	

You can save some results (swarm positions, swarm energies) in text files, 
to import them later in a spreadsheet.
A 2 dimensional representation is also saved (file visu0.txt) for post-mortem animation 

(cf the program leonardo_visu.c)										

You can try to optimize some simple functions: 
1) find DD numbers that sum to S. 
2) find DD numbers whose product is S
3) Sphere. Find DD numbers  whose sum of squares is S
4) Rosenbrock (Banana)
5) Clerc's f1 (Alpine) (WARNING: new definition, different from the one in PSOx, x<=7)
6) Griewank
7) Rastrigin
8) Fifty-fifty Problem (in particular with granul=1)
9) Ackley
10) Foxholes  2D
11) Apple trees (see apple_trees.c)
12) Graph Coloring/Frequency Assignment (see color_kit.c)
13) PSO for PSO. Recursive use to find the best model to find a solution for a given problem (or set of problems)


15) El Farol


99) MyFunction (well, yours!) (see the source file myfunction.c)

(see also the file functions.txt).

Granularity. If you _know_  that the solution, on some dimensions,  is on an "integer" point, or even a rational one,
you can give this information
Fo example, if for a given dimension, the solution is an integer, then you can set granularity to 1 for this dimension:
the process may be faster (example: functions 4 and 6, or 1 and 2 if S is an integer)

Just for fun, using the function 2 (DD-product), you can then factorize an integer number.

IMPORTANT. This program is like a Swiss knife:
 it is not at all the best for any use, but you can use it as is for a lot of problems.

Values in the model.txt file are chosen so that the _first_ run shouldn't be too bad (engineer's point of view), 
even if you could have a better _mean_ result by trying a lot of times (mathematician point of view)

Have fun!

Equations
--------
For each particle, we have

v(t+1) = alpha*v(t) + beta*(pg(t) - x(t))
x(t+1) = x(t) + v(t+1)
with
v(t) := velocity at time t
x(t) := position at time t
pg(t):= previous best position found so far in the hood (at time t), including the particle itself

alpha is adaptive (theoreticalli. In practice, it is not necessary)
beta is depending on an adaptive b

Three options:

beta = b (the normal one)
beta = rand(b_min,b)  (for test purpose)
beta = rand(0, b) (for test purpose)

Note: unlike in the complete PSO version, 
the variable pi(t):= previous best position of the particle (at time t)
is NOT used


Three kinds of adaptations:

- adaptive swarm size
- adaptive coefficients alpha and beta
- adaptive neighbourhood size for each particle


Download
--------
Well, you have it, havn't you ?
For more information and last "public" version see
 http://www.mauriceclerc.net, Math stuff for PSO
And for even more information, see the Particle Swarm Central http://www.particleswarm.net

Compiling the program
-------------------
It is written in ANSI C.

If, by chance, you have a Mac, you should use the free compiler Leonardo
http://www.dis.uniroma1.it/~demetres/Leonardo/

  Note that a PC version might come.

Using the program
-----------------
See files problems.txt (and, if you want, models.txt)
"Problems" describe the problem(s) (function, dimension, search space etc.)
"Models" describe how PSO has to work (min max hood size,min max max swarm size etc.). 
You don't _need_ to modify it, but you can.
*/

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#define E				2.718281828459045
#define	max_chaos		1
#define min_chaos		0
#define pi				3.141592653589793238462
#define Max_DD 			100 // Max number of dimensions
#define	Max_histo		10 // Max number of previous f_values stored
#define Max_hood		100 // Max size of the neighbourhood

#define Max_swarm_size 	100 // Theoretically infinite, but my computer does not like infinite...
#define Max_vel			100 // Max number of velocity components (combinatorial kit). Must be >=Max_DD
#define n_funct			15 //  ********** Modify this number if you add some functions. See also functions.txt
#define my_function		99

struct search_space	{int dim;float min[Max_DD];float max[Max_DD];float dx[Max_DD];float volume;};
struct position		{float label;int size;float x[Max_DD];float f;float ep;float prev_f[Max_histo];int adm;float X1;float X2;};
//(size, coordinates,f_value= abs(objective function value-target), potential energy, previous f_values)

struct velocity		{int size;float v[Max_vel];float ek;int v2[Max_vel];int combin;}; // (Note: v2 is useful only for combinatorial kit)
//(size, components, kinetic energy)

struct particle		{struct position pos;struct velocity vel;struct position prev_best;int H;float prev_prev_best_f;float hood_size;};
struct subswarm		{int size;int rank[Max_swarm_size];int best;int worst;};
struct swarm		{int size;struct particle part[Max_swarm_size];int best;int worst;}; 	
struct param 		{int funct;int DD;struct search_space H;float target;float eps;int batch_runs;int printlevel;float Max_Eval;int save;int N;int times;int mine;int all_diff;char data[20];float used_constr;int init_file;char init_swarm_r[20];char init_swarm_w[20];float max_fr;float parapet;int combin;};
struct vector		{int size;float x[Max_DD];};
struct vector_int	{int size;int val[Max_DD];};
struct list			{int size;float val[Max_swarm_size];};
struct matrix		{int size;int val[Max_DD][Max_DD];int nb_cont;float max_cont;}; // For square matrix of constraints
struct model		{float alpha_max;float alpha_min;float b_max;float b_min;int hood_min;int hood_max;int init_size;int max_size;int min_size;int freedom;int init_type;int H;int parallel;int local_search;int rehope;int selection;int confin;int gener_type;int move_type;int queen;int guided_rand;int rand_type;};	
struct dplus		{struct vector_int dplus[Max_DD]; int dmax; int dtot;};
struct color		{int size;int val[Max_DD];int max;int adm;};
struct color_list	{int size;int val[Max_DD];};

// Subroutines (declarations)

float			adapt_alpha(float m,struct model model);
float 			adapt_b(float m,float alpha,struct model model);
float 			adapt_hood(int sw_size, float hood_size, float dhood, int type,int hood_min,int hood_max);
struct swarm	add_particle(struct swarm sw,struct param param, struct model model,int rank);
int 			alea(int min, int max);
float			alea_density(int density_step, int d,float xmin,float xmax,int type);
int 			alea_diff(int min,int max, int num);
float 			alea_float(float min, float max);
struct position	all_different(struct position pos,struct param param);
float			apple_trees(struct position pos,int print_level); // For "apple trees" example
int				best(struct swarm sw);
int				better_than(struct particle part1,struct particle part2,float eps);			
struct particle complete_part(struct particle par, struct param param, struct model model);
struct particle constrain(struct particle part,struct param param,struct model model);
void			display_density(int dim,int density_step);
void			display_matrix(struct matrix M); // display a constraint matrix (Frequency Assignment problem
void			display_model(struct model model, int level);
void			display_param(struct param param);
void			display_position(struct position pos);
void			display_swarm(struct swarm sw, int type);
void			display_velocity(struct velocity vel);
float			distance(struct position pos1,struct position pos2);
float			distance_min(struct swarm sw, struct position pos);
float 			diverg(struct position pos);
struct position domain_pos(struct position pos,struct param param);
float			energy_p(struct particle part,float eps); // Potentiel energy of a particle
float			energy_p_swarm(struct swarm sw); // Potential energy of the swarm
float			energy_k(struct particle part,struct search_space H); // Kinetic energy of a particle
float			energy_k_swarm(struct swarm sw); // Kinetic energy of the swarm
float 			estim_eval(struct swarm sw, struct param param,struct model model);
float 			estim_eval2(struct swarm sw, struct param param,struct model model,int cycle_size);
float 			estim_eval3(int nb_past_eval,float eps);
float			estim_eval_particle(struct particle part,int nb_past_eval,float eps);
float 			eval(float total,float target);
struct position	homogen_to_carte(struct position pos);
struct particle	hood_queen(struct swarm sw,struct subswarm hood_s,struct param param,struct model model);
struct subswarm	hood_swarm(struct swarm sw,int part_rank,float eps);
struct particle	init_particle(struct param param, int type, struct model model,struct swarm sw,int rank_model);
struct swarm 	init_swarm(struct param param,struct model model);
float			improvement(struct position pos);
float 			lipschitz(struct swarm sw, int part_rank, struct subswarm hood);
float 			MAX(float a,float b);
float 			max_comp(struct position pos);
float 			MIN(float a,float b);
float 			min_comp(struct position pos);
struct particle	move_particle(struct particle part,struct particle partg,struct param param, struct model model,int rank);
struct swarm	move_swarm(struct swarm sw,struct param param, struct model model,int rank);
float			MyFunction(struct position pos,struct param param, struct model model);
float			particle_improv(struct particle part);
struct model	pos_to_model(struct position pos, struct model model);
float			PSO(int recursive,struct model model); // Useful for PSO_for_PSO (function 13)
void 			print_N_run(struct position best_result,struct param param);
struct param	read_param(FILE *f_param, FILE *f_data,int begin);
struct model	read_model(FILE *f_models);
float			regranul(float x,float granul);
struct swarm	rehope(struct swarm sw,struct param param, struct model model);
struct swarm	remove_particle(struct swarm sw,int rank, int Min_size);
void			save_energy(FILE *f_energy,struct swarm sw);
void			save_result(FILE *f_run,FILE *f_init_w,struct position best_result,struct param param,int begin, struct  model model);
void			save_swarm(FILE *f_swarm,struct swarm sw);
void			save_visu(struct swarm sw,int level,float eps);
float 			tot(struct position pos,struct param param, struct model model);
float			tot_fifty_fifty(struct position pos);
void			update_density(struct position pos,struct search_space H,int step);
struct velocity	vel_granul(struct velocity vel,struct search_space H);
int				worst(struct swarm sw);

// Global variables

double			almostzero=0.0000001;  //to avoid overflow by dividing by too small value;
int				AS=-3; // Arbitrary value. Means "Assigned"
int				batch_runs; 
int				begin;
float			best_eval[Max_histo][2][2];
struct position	best_result[2]; // Two levels, for eventual recursive mode (PSO_for_PSO)
int				chance[2]; // Flag, for solution found by chance
float			chaos=(float)0.6; // Init value for chaotic function
clock_t			clock_tick;
int				density[10][Max_DD]={0}; // For Guided Random Generation. Cf init_particle()
int				density_step; // Number of points on each dimension to define "cells" for Guided random generation 
struct dplus	dplus; // To store a graph (for each node, list of neighbours). Just for Graph Coloring
float			duration;
float			duration_tot;

float			E0;
float			E_k; // Kinetic energy of the swarm
float			E_p; // Potential energy of the swarm
float			eval_f[2],eval_f_tot[2];
int				failure_tot[2];
char			functions[30][30]; // Function names
int				H; // Flag for Coloring problem. Indicates what kind of projection to do
float			label[2]; // To labellize particles
int				level; // For recursive mode
struct matrix	M;
int				Max_hood_size; // Just for info
int				max_rand;
int				Max_swarm; // Just for info
float			Mean_swarm; // Just for info
struct model	models[2]; // Model level 0 and Initial model level 1 for recursive mode (PSO_for_PSO)
struct model	model2; // Modified model for recursive mode (PSO_for_PSO)
float			move[2]; // For statistical success estimations
int				n_add;
int				n_rand; // Type of pseudorandomness (see alea_float())
int				n_remove;
int				NA=-32000;// Arbitrary value. Means "non assigned"
int				NO=-2; // Arbitrary value. Means "no change"
struct param	param2; // For recursive mode (PSO_for_PSO)
float			rand_list[1000];
int				rand_type; // Type of randomness. Cf alea_float()
int				recursive=0;
int				self_better[2]; // Just for stat
struct	swarm	sw;
float			two_pi;
float			volume_checked[2];

// For Leonardo visualization

float			X1,X2;
FILE *f_visu0; // For visualization (X1,X2,value) (level 0)
FILE *f_visu1;

//--------
FILE *f_init_r; // (optional) to Read an initial swarm
FILE *f_init_w; // to Write an initial swarm for future use

FILE *f_data; // For additional data, defining the problem (see Coloring/Frequency Assignment)
FILE *f_energy; // To save swarm's energy
FILE *f_functs; // Function names
FILE *f_models; // PSO models (level 0, and 1 if recursive)
FILE *f_move_color; // For Coloring problem. Save particle positions in a 2D space (colors, constraints)
FILE *f_param; 	// Parameter file
FILE *f_rand;   // List of random numbers in [0,1]
FILE *f_run; 	// To save the run
FILE *f_swarm;  // To save swarm
FILE *f_synth; // 
FILE *f_trace;

// External subroutines

#include "color_kit.h" // For Graph coloring problems
#include "combinatorial_kit.h" // For combinatorial problem
#include "color_kit.c"
#include "combinatorial_kit.c"
#include "myfunction.c" // Write your own objective function in this file.
#include "apple_trees.c" // For "apple trees" example
#include "hyper_vol_1_300.txt" // List of unit hypersphere volumes for dimensions 1,2,...300

/* =============================================================================== */
void main()
{
int				i;
float			Mean_eval;
char 			mr[1];

f_functs=fopen("functions.txt","r");
f_models=fopen("models.txt","r");

two_pi=(float)(2*pi);

/* Initialize pseudorandom numbers  (see alea_float() ), just in case
   you use this option (see models.txt, model.rand_type=1)
*/ 

f_rand=fopen("rand_list.txt","r");
fscanf(f_rand,"%i",&max_rand);

for (i=0;i<max_rand;i++)
{
	fscanf(f_rand,"%f",&rand_list[i]);
}
fclose (f_rand);

// Initialize function names. Just for display

for (i=1;i<=n_funct;i++)
	{
	fscanf(f_functs,"%s\n",&functions[i]);
	printf("\n %i %s",i,functions[i]);
	}
strcpy(functions[99],"My function ");
printf("\n-----------------------------------\n");

fclose (f_functs);

// Model(s)
printf("\n Read Model 0");
models[0]=read_model(f_models); // Level 0 model
printf("\n Read Model 1 (in case you use recursive PSO)");
models[1]=read_model(f_models); // Level 1 model (useful only for recursive use  (see param.funct=13 in tot() )
fclose (f_models);
display_model(models[0],0);

//===================  Other files  (they must be open during the whole process)
f_visu0=fopen("visu0.txt","w"); // For post mortem visualisation
f_visu1=fopen("visu1.txt","w"); // The same, in case of recursive PSO

f_energy=fopen("energy.txt","w");
f_move_color=fopen("f_move_color.txt","w");
f_param=fopen("problems.txt","r");

f_run=fopen("run.txt","w");
f_swarm=fopen("swarm.txt","w");
f_synth=fopen("synth.txt","w");
f_trace=fopen("trace.txt","w");
//-----------------


level=0;
best_result[0].size=0; // Still no "best result" at all


Mean_eval=PSO(0,models[0]); // ******* CORE OF THE PROGRAM *******


printf("\n Any letter and Return: ");
scanf("%s",&mr); 
return;
}

/*============================ S U B R O U T I N E S =====================================*/
/* Except the first one, they are in alphabetical order                                   */

float PSO(int recursive,struct model model)  // Main routine. Separated for optional recursive use
{
/* Main subroutine */

int				add[Max_swarm_size];
int				best;
struct color	color;
float 			currenteps;
int				cycle,cycle_size;
float			dhood;
float			enough_improv;
float			eval_f_max;
float			eval_f_prev;
float			eval_f_sup=0; // Just for particular tests
float			eval_run[100]; // Just for information
struct subswarm	hood_s;
int				hood_size;
int				i;
int				i_step;
int				j;
float			improv;
int 			Iter;
float			mean_eval;

float			min_eps;
float			nb_eval_max;
float			nb_eval_min;
float			nb_no_progress;
int				nb_rehope;
struct param 	param={0};
float			previous_eps;
struct particle	queen;
int				rank;
int				remove[Max_swarm_size];
int				size0;
float			stop;
clock_t 		ticks0,ticks1, ticks2;
int				times;
int				worst;
float			zzz;

if (param.printlevel>0)
	printf("\n Level %i. PSO", level);

rand_type=model.rand_type; // Global variable. Type of randomness (see alea_float())	
n_rand=0; // Number of times alea_float() is called
stop=40000; /* Arbitrary stop criterion (objective function evaluations)
              You may remove it, for there is also an adaptive automatic stop criterion
 			*/
zzz=0;
nb_eval_max=0;
nb_eval_min=(float)-NA; 

begin=0;

if (level==0)
	{
	save_result(f_run,f_init_w, best_result[level],param,0, model); // Titles
	fprintf(f_energy,"eval_f swarm_size potential_energy kinetic_energy\n");
	}

// Parameters of the problem(s)
read_param:


if (recursive<2)
{
	param=read_param(f_param,f_data,begin); // READ THE PROBLEM DESCRIPTION

	if (param.funct==13) display_model(models[1],1); // If PSO for PSO, display the second model
	
	// Check improvement every cycle_size time steps

	cycle_size=model.min_size; // Initial value
	
	// Just to avoid too long computation
	param.parapet=(float)(model.hood_min*model.max_size*model.hood_min);
	
	// Combinatorial problems are more difficult
	if (param.combin>0 || param.funct==12) param.parapet=param.parapet*(model.max_size-1)/2; 
	

		
	if (param.printlevel>0)
	{
		printf("\n=======================================\n Maximum swarm size: %i",model.max_size);
		printf("\n In any case, no more than %.0f evaluations without improvement",param.parapet);
	}

	param2=param; // Save for future use in recursive mode
	model2=model;
	display_param(param);	
}
else
{
	if (level>0)
	{
		param=param2;
	
		if (best_result[level-1].size>0)
		{
			model=pos_to_model(best_result[level-1],model2); // The new model at this second level is the best "position"
												// at the previous level
		}
		else
		{
			model=model2;
		}
	}											
}

if (begin==0) {batch_runs=param.batch_runs;begin=1;}

times=0; // you can run several times the same problem (parameter param.times)
ticks0=clock();
duration_tot=0;
volume_checked[level]=0; // Just for information and estimation of time still needed

min_eps=1/almostzero;

//========================================================= LOOP on problems to solve
times_loop: 
label[level]=0; // Label of the first particle. Just for later visualisation
nb_rehope=0; // Number of times Rehope process has been launched

failure_tot[level]=0; // Number of failures


if(param.printlevel>0)
{
	printf("\n\n--------------\n Level %i. %s %iD",level,functions[param.funct],param.DD);
	printf("  Run %i/%i",times+1,param.times);
}

ticks1=clock(); // Just for information. To evaluate processor time needed

chance[level]=0; // Equal to 1 if a solution is found
eval_f_tot[level] =0; // Number of objective function evaluations
eval_f_prev=0;
eval_f_max=(float)(1/almostzero); // At the very beginning, choose a very high max number of evaluations
eval_f[level]=0; // Number of evaluations counter
nb_no_progress=0;// If this number becomes > parapet, then => Failure
Iter=1;		// Number of iterations. Just for information
self_better[0]=0; // Just for stats
self_better[1]=0;
density_step=model.guided_rand;
dhood=1;// Increment for adaptive hood size
param.N=model.init_size;// Normally we begin with just a few particles, but you can change it (see models.txt)
H=1; // Just for Coloring problem. Projection will be done first on H1 (minimize constraints)
rank=-1;
if (model.freedom==6) rank=0; // Special case "indepnedent dimensions"

sw=init_swarm(param, model); 	// ******** SWARM INITIALIZATION *********

if (param.save==2) save_visu(sw,level,param.eps);
	    
param.N=sw.size; // Useful if the initial swarm has been in fact read on a file, at least partly 
Max_swarm=sw.size; // Just for information
Mean_swarm=(float)sw.size; // Just for information
Max_hood_size=0;// Just for information


if (chance[level]==1) goto end_by_chance; // Sometimes, you find a solution by chance at the very beginning
				
best_result[level]=sw.part[sw.best].prev_best; // Keep trace of the best result found so far
previous_eps=best_result[level].f;
currenteps=previous_eps;

for (i=0;i<Max_histo;i++) {best_eval[i][0][level]=eval_f_tot[level];best_eval[i][1][level]=currenteps;}

// Initial value for "enough improvement", to begin
enough_improv=(sw.part[sw.worst].prev_best.f-sw.part[sw.best].prev_best.f)/sw.part[sw.worst].prev_best.f;

if (param.printlevel>1) 
{
	printf("\n  Best result after %.0f evaluations ",eval_f_tot[level]);
	display_position (best_result[level]);
	printf("\n------ MOVING the swarm (size %i)",sw.size);
}						

cycle=0;  // Number of iterations since last adaptive update


do_Iter: //termination criterion is (currenteps<=eps or eval_f>=eval_f_max or eval_f_tot>stop)
						
	Iter=Iter+1; // number of time steps. Just for information
	move[0]=0;move[1]=0; // Number of moves, number of succesful moves
	if (param.save==1) 
	{
		save_swarm(f_swarm,sw);	
		save_energy(f_energy,sw);
	}

	eval_f_prev=eval_f_tot[level];
	sw=move_swarm(sw,param, model,rank); 					// *** HERE ! THE SWARM MOVES ! **********
	
	if (param.save==2) save_visu(sw,level,param.eps);
	if (param.save==3) fprintf(f_trace,"\n %.0f %.0f",move[0],move[1]);

	if (best_result[level].f<min_eps) min_eps=best_result[level].f;
	if(chance[level]==1) goto end_by_chance; // A solution has been found, maybe before all particles have moved
	
	if (rank>=0)
	{
		if (fabs (best_result[level].x[rank]-sw.part[sw.best].prev_best.x[rank])<param.eps)
		{
			rank=rank+1; if (rank==param.DD) rank=0; // Special case "independent dimensions"
		}
	}

	// Eventually update the best result
	if (sw.part[sw.best].prev_best.f<best_result[level].f)
	{
		best_result[level]=sw.part[sw.best].prev_best;// Abs(target-tot) for the best particle
		currenteps=best_result[level].f;	
	}
	
		
	if (param.printlevel>2) display_swarm(sw,1); 
	if (param.printlevel>1)
	{
		printf("\n  Best result after %.0f evaluations ",eval_f_tot[level]); 
		display_position(best_result[level]);
	}

	// For recursive PSO, systematically save intermediate results	
	if (param.funct==13)	 
	{	
		save_result(f_run,f_init_w,best_result[level],param,1, model);
	}

	if (param.printlevel>2)
	{
		printf("\n Energies (potential, kinetic):  %f  %f",E_p,E_k);
	}

	if (eval_f_tot[level]>stop) // Arbitrary stop criterion
	{
		failure_tot[level]=failure_tot[level]+1;
		printf("\n FAILURE eval_tot  %.0f > max= %.0f",eval_f_tot[level],stop);
		goto the_end;
	}

	if (eval_f[level]>eval_f_max) // Adaptive stop criterion
	{
		failure_tot[level]=failure_tot[level]+1;
			printf("\n FAILURE eval  %.0f > max=%.0f",eval_f[level],eval_f_max); 
		goto the_end;
	}

	                                // ***********  ADAPTATION(S)  (begin) *************
	cycle=cycle+1;
	cycle_size=sw.size;

	if (2*cycle<cycle_size) goto end_adapt; // Adaptation is not made at each iteration
										 // just from time to time (after "cycle_size" iterations) 
										// Note that cycle_size is not constant
									
		
		/* If improvement, "bad" particles are not useful => remove them
		   If deterioration, it is worthwhile to generate a particle
		*/
		
		// Store best evaluation  (just for info and prediction purpose		
		for (i=0;i<Max_histo-1;i++)
		{
			best_eval[Max_histo-i-1][0][level]=best_eval[Max_histo-i-2][0][level];
			best_eval[Max_histo-i-1][1][level]=best_eval[Max_histo-i-2][1][level];
		}
		best_eval[0][0][level]=eval_f_tot[level];
		best_eval[0][1][level]=currenteps;

		// Some inits for adaptation
		n_add=0; // Number of particles added. Just for information
		n_remove=0; // Number of particles removed. Just for information
		size0=sw.size;
		
		if (model.hood_min==0) // Option (cf models.txt): Neighbour = whole swarm
		{
			hood_size=sw.size;
		}
	
		for (i=0;i<size0;i++) //  Reinit add[] and remove[], status of particles to add or remove
		{
			add[i]=0;
			remove[i]=0;
		}
		
/*
		for (i=0;i<size0;i++) // Save info about evolution of neighbourhood sizes
		{
			fprintf(f_run,"\n %.0f %f %f", sw.part[i].pos.label, eval_f_tot[level],sw.part[i].hood_size);
		}
*/
			
		i=0;
		next_hood: // Loop on neighbourhoods, to check them 
		{
			hood_s=hood_swarm(sw,i,param.eps); // Generate the neighbourhood
			best=hood_s.rank[hood_s.best];
			worst=hood_s.rank[hood_s.worst];
			
			// Evaluate local improvement
			
			improv=particle_improv(sw.part[best]);

			if (improv<enough_improv) // If not enough improvement
			{ 
				if(model.queen==1 && sw.size<Max_swarm_size)// Try the local queen
				{
					queen=hood_queen(sw,hood_s,param,model);
			
					if (queen.pos.f<=param.eps)
					{
						best_result[level]=queen.pos;
						chance[level]=1;
						printf("\n Solution thanks to a queen");
						goto end_by_chance;
					}

					if( queen.pos.f<sw.part[worst].prev_best.f) 
					{
						//printf("\n queen is better");								
						sw.part[sw.size]=queen;  // add the queen
						sw.size=sw.size+1;
						n_add=n_add+1; 
						goto next_part;
					}
				}

				add[best]=1;
				// Adapt hood size (increased because not enough improvement)
				sw.part[best].hood_size=adapt_hood(sw.size,sw.part[best].hood_size,dhood,1,model.hood_min,model.hood_max);
				goto next_part;
			}

			if (improv>=enough_improv) // If  enough improvement, remove the worst particle
			{
				remove[worst]=1;
				// Adapt hood size (decreased, because enough improvement)
				sw.part[best].hood_size=adapt_hood(sw.size,sw.part[best].hood_size,dhood,0,model.hood_min,model.hood_max);
				goto next_part;
			}


next_part:
			// Just once by complete neighbourhood
			if (model.hood_min>0) hood_size=(int)sw.part[i].hood_size; 
			i_step=(int)(hood_size+(float)hood_size/2 +1); 
			//i_step=1; // For each particle. For test purpose
			i=i+i_step;
			if (i<size0) goto next_hood;
		}


	size0=sw.size;
	
	// WARNING. Do add before to remove. Rank i is used in gener_particle_color
		for (i=0;i<size0;i++) // Adding
		{
			if (add[i]==1)
			{
				sw=add_particle(sw,param, model,i);  // add a particle 
				if(chance[level]==1) goto end_by_chance;

				// enough_improv is probably too "ambitious" => decreasing
				//enough_improv=enough_improv/(1+1/(float)sw.size);
				//enough_improv=enough_improv/(2-1/((float)sw.size)+1);	
				enough_improv=enough_improv/(2-1/exp((float)sw.size));
				
						
			}
		}
			
		for (i=0;i<size0;i++) // Removing
		{
			j=size0-i-1;
			if (remove[j]==1)
			{
				sw=remove_particle(sw,j, model.min_size);

				// enough_improv is reasonable. One try to increase it
				//enough_improv=enough_improv*(1+1/(float)sw.size);
				//enough_improv=enough_improv*(2-1/((float)sw.size)+1);
				enough_improv=enough_improv*(2-1/exp((float)sw.size));
					
			}
		}

		param.N=sw.size; // For eventual re-initialisation
		Max_swarm=(int)MAX((float)Max_swarm,(float)sw.size); // Just for information
		
		cycle=0;

		eval_f_max=estim_eval(sw,param,model); // **** HERE THE NUMBER OF EVALUATIONS TO DO IS RE-EVALUATED ***
//printf("\n estim1 %.0f",eval_f_max);		
		//eval_f_max=estim_eval2(sw,param,model,cycle_size);
//printf("\n  estim2 %.0f",eval_f_max);
	//eval_f_max=estim_eval3(3,param.eps);
//printf("\n  estim3 %.0f",eval_f_max);
		eval_f[level]=0;
	 
			

		if (param.printlevel>0)
		{
			printf("\n");
			printf(" Eval: %.0f",eval_f_tot[level]);
		 	printf("  Estim.: %.0f",eval_f_max);
			printf(" Result: %.8f",currenteps);
			printf(" (part.  %.0f).",sw.part[sw.best].pos.label);
			printf(" Swarm size %i",sw.size);
			printf(" (+ %i, -%i)",n_add,n_remove);
		}
		
		
		end_adapt:;
		
		                     // ---------------------------  ADAPTATION(S)  (end)	

		Mean_swarm=Mean_swarm+sw.size;
		
		if (currenteps>=previous_eps)
		{
			if (currenteps>previous_eps) // Normally, it can't be the case, but there may be a bug...
				printf("\n WARNING. Non monotonous convergence. eps= %f (was %f)",currenteps,previous_eps);	
			nb_no_progress=nb_no_progress+1; // Number of iterations without any improvement

			if (model.rehope>0 && nb_no_progress>param.parapet) 
			{
				sw=rehope(sw,param, model); // REHOPE
				
				if(chance[level]==1) goto end_by_chance; // A solution has been found, maybe before all particles have moved 
				
					best_result[level]=sw.part[sw.best].prev_best;// Abs(target-tot) for the best particle
					currenteps=best_result[level].f;
					
				nb_rehope=nb_rehope+1;
				printf("%i/%i",nb_rehope, model.rehope);

				if (nb_rehope>model.rehope)
				{
					failure_tot[level]=failure_tot[level]+1;
					goto the_end;
				}

				eval_f_max=(float)(1/almostzero);  
				nb_no_progress=0;
			}
		}
		
		previous_eps=currenteps;			
		goto do_Iter; // Iterates till a criterion is met 
			
end_by_chance:

currenteps=best_result[level].f;
if(param.printlevel>0)
	printf("\n CHANCE !");

the_end: // ---------------------------- END

ticks2=clock();
clock_tick=ticks2-ticks1; // Processor time needed

if (param.funct==12) // For Coloring/ Frequency assignment, simplify best_result to display/save
	best_result[level]=color_shift(best_result[level]);

if (level==0)
{						
	save_result(f_run,f_init_w,best_result[level],param,1, model);
}

if(param.save==1) save_swarm(f_swarm,sw);
	
	duration = (float)(clock_tick/CLOCKS_PER_SEC);
	duration_tot=duration_tot+duration;
	eval_run[times]=eval_f_tot[level];
	
	if (level==0)
	{
		printf ("\n Run %i => eps %f, eval: %.0f, duration: %f",times+1,currenteps,eval_f_tot[level],duration);
		printf(" (by part. %.0f)",best_result[level].label);
		printf("\n Max swarm size = %i.",Max_swarm);
	

		if (times>0) printf(" Mean swarm size =  %.0f.",Mean_swarm/Iter);
		printf(" Max neighbourhood size = %i",Max_hood_size);
		
		if (model.move_type==2)  // Stats about  this option (self move), if used
		{
			printf(" Self better  %.2f ",(float)self_better[1]/(float)self_better[0]);
		}

		if (param.funct==12)  // Special case Coloring problem
		{
		// Better to verify ...
			color=convert_position_to_color(best_result[level]);
			i=check_color(color,M,dplus);
			printf(" Unsatisfied constraint(s): %i",i);
			i=diff_color(best_result[level]);
			printf(", %i colors",i);
		} 
	}

zzz=zzz+eval_f_tot[level];

if (eval_f_tot[level]>nb_eval_max) nb_eval_max=eval_f_tot[level];
if (eval_f_tot[level]<nb_eval_min) nb_eval_min=eval_f_tot[level];

if(param.printlevel>0)			
	print_N_run(best_result[level],param);
	
if (param.printlevel>1) display_swarm(sw,1);
			
	times=times+1;
	if (times<param.times) goto times_loop;
	
		ticks2=clock();
		mean_eval=zzz/param.times;
		Mean_swarm=Mean_swarm/Iter; // Just for information, mean swarm size
		
		if (level==0)
		{
			fprintf(f_synth,"\n");
			fprintf(f_synth," %2i",param.funct);
			fprintf(f_synth," %2i",param.DD);
			fprintf(f_synth," %4.3f",param.target);
			fprintf(f_synth," %4.10f",param.eps);
			fprintf(f_synth," %4i",param.times);
			fprintf(f_synth," %.0f",zzz); // Mean number of evaluations

			zzz=0;

			for (i=0;i<param.times;i++)

			{

				zzz=zzz+(eval_run[i]-mean_eval)*(eval_run[i]-mean_eval);

			}

			zzz=sqrt (zzz/param.times);



			fprintf(f_synth," %f",zzz); // Standard deviation




			fprintf(f_synth, " %.0f %.0f",nb_eval_min,nb_eval_max);

			fprintf(f_synth, " %.0f",min_eps);
			
			printf("\nMEAN NUMBER OF EVALUATIONS: %.0f",mean_eval); // Mean number of evaluations
			
			printf(" +- %f",zzz); // Standard deviation
			printf(" [%.0f %.0f]",nb_eval_min,nb_eval_max);

			printf(" Min eps: %f",min_eps);
			printf("\nTOTAL NUMBER OF FAILURES: %i", failure_tot[level]);
			if (eval_f_sup>0) printf(" (after %.0f evaluations)",eval_f_sup);

			duration = duration_tot/(float)param.times;
			printf(" / MEAN TIME %4.4f",duration);

			fprintf(f_synth," / Time %4.4f",duration);

			if(model.guided_rand>0 && param.printlevel>0)
				display_density(param.DD,density_step);	
			
			if (param.printlevel>0)
			{
				printf("\n Evolution summary (eval. nb, best error value)");
				for (i=0;i<Max_histo;i++)
					printf("\n %.0f %f",best_eval[Max_histo-i-1][0][level],best_eval[Max_histo-i-1][1][level]);
			}
	
		}
		
		batch_runs=batch_runs-1;
		if (batch_runs>0) goto read_param;
		
		if (param.funct==12)  // Special case Coloring problem
		{
			fprintf(f_init_w,"\n-1"); // End of file
			fclose(f_init_w); // An init file  has been written.
		} 
		
		fprintf(f_visu0,"%i\n",-1); // End of visu file level 0
		fprintf(f_visu0,"\n %.0f evaluations \n",zzz);
		
		fprintf(f_visu1,"%i\n",-1);
		fprintf(f_swarm,"%i\n",-1); // End of save swarm file
		
return zzz;

}
// ----------------------------------------------------------------------------- ADAPT_ALPHA
float adapt_alpha(float m,struct model model)
{
float 	alpha,alpha_inf,alpha_sup;
float	k;
float lambda;
float	mu;

//return alea_float(model.alpha_min,model.alpha_max); // Random, for test purpose

alpha_inf=model.alpha_min;
alpha_sup=model.alpha_max;

if (alpha_sup<=alpha_inf) return alpha_inf;

alpha=(alpha_inf+alpha_sup)/2 + m*(alpha_sup-alpha_inf)/2;
return alpha;

/*
k=1;
mu=(alpha_inf+alpha_sup)/2;
lambda=(float)(mu-(alpha_sup-alpha_inf)/(exp(k)+1));
lambda=(alpha_sup-mu)/lambda;
alpha=(float)(mu+lambda*(alpha_inf+(1-1/(exp(k*m)+1))*(alpha_sup-alpha_inf) - mu));
return alpha;
*/
}

// ----------------------------------------------------------------------------- ADAPT_B
float adapt_b(float m, float alpha, struct model model)
{
float	b;
float	b_inf;
float	b_sup;
float	k;
float	lambda;
float	mu;

if (model.b_max<=model.b_min) return model.b_min; // Constant value


b_sup=MIN(model.b_max,(float)(alpha+1+2*sqrt(alpha))); // Max value for sure convergence
b_inf=MAX(model.b_min,(float)(alpha+1-2*sqrt(alpha)));

if (model.freedom==2) return alea_float(b_inf,b_sup);

b=(b_inf+b_sup)/2+m*(b_sup-b_inf)/2; 
//printf("\n m %f,  b %f",m,b);

//return b;

return alea_float(b_inf,b); // Partly random, for test purpose

// Partly random, for test purpose
b_inf=MAX(b_inf,0.8*b);
b_sup=MIN(1.2*b,b_sup);
//b=alea_float(b_inf,b_sup); 


return b;

/*
k=1;
mu=(b_inf+b_sup)/2;
lambda=(float)(mu-(b_sup-b_inf)/(exp(k)+1));
lambda=(b_sup-mu)/lambda;
b=(float)(mu+lambda*(b_inf+(1-1/(exp(k*m)+1))*(b_sup-b_inf) - mu));

//printf(" b %f",b);
return b;
*/
}

// ----------------------------------------------------------------------------- ADAPT_HOOD
float adapt_hood(int sw_size, float hood_size, float dhood, int type,int hood_min,int hood_max)
{
//float 	delta;
float 	new_hood_size;
float	mu=0.5;
float	N;

if (sw_size==hood_min) return (float)hood_min;
if (hood_max<=hood_min) return (float)hood_min;

N=(float)sw_size;

if (type==1)// Increase hood size
{
	/*
	delta=dhood*(1-(int)hood_size/(float)sw_size);
	new_hood_size=hood_size+delta;
	new_hood_size=MIN(new_hood_size,(float)hood_max);
	*/
	new_hood_size=hood_size+(N-hood_size)/(N-1);
}		
else // Decrease hood size
{
	/*
	delta=dhood*(int)hood_size/(float)sw_size;
	new_hood_size=hood_size-delta;
	new_hood_size=MAX(new_hood_size,(float)hood_min);
	*/
	new_hood_size=hood_size-(hood_size-1)/(N-1);;
}

if ((int)new_hood_size>Max_hood_size) Max_hood_size=(int)new_hood_size; // Max hood size, Just for info

//printf("\n type %i,sw size %i, hood %f, new hood %f",type, sw_size, hood_size,new_hood_size);

return new_hood_size;
}					
// ----------------------------------------------------------------------------- ADD_PARTICLE
struct swarm add_particle(struct swarm sw,struct param param, struct model model,int part_rank)
{
/*
Generate a new particle
part_rank is the rank of the particle which launched this generation
The bigger the swarm the less you can easily add a particle

*/
int				i;
struct swarm 	swt;

//if (E_p+E_k>=E0) return sw;
if (sw.size>=model.max_size) return sw;



n_add=n_add+1;
swt=sw;

i=sw.size;
//printf("\n Add a particle at rank %i, from particle %i",	i, part_rank);

if (model.gener_type==0 || param.funct!=12) // Normal case (not coloring problem)
{
	swt.part[i]=init_particle(param,1, model,sw,part_rank); // New particle
}

if (model.gener_type==1 && param.funct==12) // Special case for coloring problem
{
	swt.part[i]=gener_particle_color(sw,part_rank,param, model); // New particle
}


E_p=E_p+swt.part[i].pos.ep;
E_k=E_k+swt.part[i].vel.ek;

swt.size=sw.size+1;

return swt;
}

// ----------------------------------------------------------------------------- ALEA 
int alea(int min, int max) /* Random integer number between min and max */
{
int		ir;
float 	r;

r=alea_float(0,1);

ir=(int)(min+r*(max+1-min));
if (ir>max) ir=max;
return ir;
}

// ----------------------------------------------------------------------------- ALEA_DENSITY
float	alea_density(int density_step, int d,float xmin,float xmax,int type)
{
/* 
Guided Random Generation
Maurice.Clerc@WriteMe.com 2001-09-20

Generate a random number for dimension d, according to the density of the positions already evaluated
Note: 
density[density_step][number_of_dimensions] is a global variable.
     Keep trace of the number of evaluated positions, for each dimension and each interval
density_step is a global variable. Number of intervals for dimension d
d is the dimension currently considered
xmin, xmax : min and max possible values for each dimension d
type = 0  => more probable where density is low
type = 1  => more probable where density is high
*/

float	dpr;
float	dx;
int		k;
int	i,j;
float	pr[10];
float	pr_c[11];
float	r;
float	sigma1, sigma2;


if (density_step<=1) return alea_float(xmin,xmax);

if (density_step>=10) printf("\n ERROR alea_density. density_step too high");

// Look for the max value
sigma1=0;
for (i=0;i<density_step;i++)
{
	if (sigma1<=(float)density[i][d]) sigma1=(float)density[i][d];	
}

if (sigma1==0) return alea_float(xmin,xmax);

k=1; // You may increase it for stronger effect

// Attraction/Repulsion function. YOU MAY CHANGE IT
if (type==0)// Attraction function (decreasing function of the density)
{	
	sigma2=0;
	for (i=0;i<density_step;i++)
	{
		pr[i]=(float)pow (sigma1-density[i][d],k);
		sigma2=sigma2+pr[i];
	}
}
else // Repulsion function (increasing function of the density)
{
	sigma2=0;
	for (i=0;i<density_step;i++)
	{
		pr[i]=(float)pow(density[i][d],k);
		sigma2=sigma2+pr[i];
	}
}


if (sigma2<=0) return alea_float(xmin,xmax);


// Cumulated
pr_c[0]=0;
pr_c[1]=pr[0];

for (i=2;i<=density_step;i++) 
{
	pr_c[i]=pr[i-1]+pr_c[i-1];
}

// Find a random number
r=alea_float(0,sigma2); 

// Modify it according to the attraction/repulsion function
dx=(xmax-xmin)/(float)density_step;

for (i=0;i< density_step;i++)
{
	j=density_step-i-1;
	if (pr_c[j]<=r)
	{
		dpr=pr_c[j+1]-pr_c[j];
		if (dpr==0) goto next_i;
		dpr=(r-pr_c[j])/dpr;
		r=xmin + dx*(j + dpr);
		return r;
	}
next_i:;
}

printf("\n ERROR alea_density. r = %f, j = %i, pr_c[j] = %f, sigma2 = %f",r,j,pr_c[j],sigma2);

printf("\n infinite waiting loop");infinite:goto infinite;
}
/*------------------------------------------------------------------- ALEA_DIFF */
int alea_diff(int min,int max, int num) /* Generate randomly an integer number # num */
{
int	temp1,temp2;

if (num==min)
	{
	temp1=alea(min+1,max);
	return temp1;
	}
	
if (num==max)
	{
	temp1=alea(min,max-1);
	return temp1;
	}

temp1=alea(min,num-1);
temp2=alea(num+1,max);

if (alea(0,100)<50)
	{
	return temp1;
	}
else
	{
	return temp2;
	}
}


// ----------------------------------------------------------------------------- ALEA_FLOAT 
float alea_float(float min, float max) /* Random  number between min and max */
{
float 	r;

/*
Note: rand_type is a global variable (cf models.txt )
0  => Use the "random" numbers generator
1  => Use the file of "random" numbers
2  => Use chaos. Seems not as good as randomness, even pseudo
*/
switch (rand_type)
{
	case 1:
	// Using this, you have the same result on different machines
	// BUT, sometimes performances are bad, probably because 
	// you  reuse the same pseudorandom numbers (possibility of cycles)

	r=rand_list[n_rand];
	n_rand=n_rand+1;
	if (n_rand>=max_rand-1)
	{
		printf("\n (reset rand_list after %i random numbers read)",max_rand);
		n_rand=0;
	}
	break;
	
	case 2: // The simplest "deterministic" way. But seems quite bad
		retry:
		chaos=4*chaos*(1-chaos); // Global variable
		// Distribution is well known to be not uniform (parabolic)
		// we keep only intermediate values
		if (chaos<0.2) goto retry; 
		if (chaos>0.8) goto retry;	
		r=(chaos-0.2)/0.6;
		break;
		
	default: // The classical and apparently the best one
		r=RAND_MAX; // Normally, RAND_MAX = 32767 = 2^31-1 
		r=rand()/r;
		break;
	}
	
	r=min+r*(max-min);
	return r;
	
}

// ----------------------------------------------------------------------------- ALL_DIFFERENT
struct position	all_different(struct position pos,struct param param)
{ 
/*
 Modify the position so that all components are different
 Should be used only when granularity>O
 Useful for some combinatorial problems, like Knapsack or Fifty-fifty
*/

int				d,d1;
float			dx;
int				k;
float			max_loop;
float			nloop;
struct position	post;

post=pos;

for (d=1;d<pos.size;d++) // For each component ...
{
	if (param.H.dx[d]>0) {dx=param.H.dx[d];} else {dx=param.eps;}
	max_loop=1+(param.H.max[d]-param.H.min[d])/dx;
	k=1;
	nloop=0;
	loop1:
	//... if component # of all previous, OK, ...
	for (d1=0;d1<d;d1++)
	{
		if (post.x[d]==post.x[d1]) // ... else, modify slightly and retry
		{
			loop2:
			post.x[d]=pos.x[d]-k*dx;
			if (k<0) {k=-k+1;} else {k=-k;}
			if (post.x[d]<param.H.min[d]) goto loop2;
			if (post.x[d]>param.H.max[d]) goto loop2;
				nloop=nloop+1;
				if (nloop==max_loop) 
				{
					printf("\n WARNING. Some components are equal (%i and %i) Impossible to modify this particule:",d1,d);
					display_position(post);
					return post;
				}
				goto loop1;
		}
	}
}
return post;
}

// ----------------------------------------------------------------------------- BEST
int best(struct swarm sw)// Find the "previous best" particle in the whole swarm
{
int i;
int	sw_best;

sw_best=0; // First, arbitrary value for the rank of the best "previous best"

for( i = 1;i<sw.size;i++)  // Look for a better one
	{
	if (sw.part[i].prev_best.f<sw.part[sw_best].prev_best.f)
		sw_best=i;
	}	
return sw_best;
}

// ----------------------------------------------------------------------------- BETTER_THAN
int	better_than(struct particle part1,struct particle part2,float eps)
{ // return 1 if part1 is better than part2 (or same quality)

if (part1.prev_best.f>part2.prev_best.f+eps) return 0; // pos2 is better
if (part1.prev_best.f<part2.prev_best.f-eps) return 2; // pos1 is better

if ((part1.	prev_prev_best_f-part1.prev_best.f)>=(part2.prev_prev_best_f-part2.prev_best.f)) return 1;

return 0;

}

// ----------------------------------------------------------------------------- COMPLETE_PART 
struct particle complete_part(struct particle par, struct param param, struct model model)	
 // WARNING: position and velocity must be already initialized. Complete the particle
{
int				d;
struct particle	part;
float			total;

part=par;

part.pos.label=label[level];
part.prev_best.label=label[level];
label[level]=label[level]+1;

part.pos.size=param.DD;
part.vel.size=param.DD;
part.hood_size=(float)model.hood_min;


if (param.printlevel>2)
printf("\n complete_part, label %.0f, H %i",part.pos.label, part.H);

part=constrain(part,param,model); // Keep in the search space

	
if (param.all_diff>0) // Eventually, modify so that all components are different
		part.pos=all_different(part.pos,param);
	
total=tot(part.pos, param, model);       //evaluation of the position
part.pos.f=eval(total,param.target);

part.pos.X1=X1; // For Leanordo visualization
part.pos.X2=X2;

part.prev_best=part.pos; // previous best = current position

for (d=0;d<Max_histo;d++) // previous f-values for the last  steps 	
{
	part.pos.prev_f[d]=part.pos.f;
}
	
part.prev_prev_best_f=part.pos.f;
part.pos.ep=energy_p(part,param.eps);	

if(part.pos.f<param.eps) 
	{
	best_result[level]=part.pos;
	chance[level]=1;
	}

part.vel.combin=param.combin; // Remember what kind of velocity is used (combinatorial or not)
part.vel.ek=energy_k(part,param.H); // Kinetic energy

return part;
}

// ----------------------------------------------------------------------------- CONSTRAIN
struct particle constrain(struct particle part,struct param param, struct model model)
{ /*
 Keep the particle in the definition domain 
*/
int				cmax;
int				d;
struct particle partt;

float			xmax;
float			xmin;	

//if (param.combin>0) return part;

cmax=(int)param.max_fr;
partt=part;

if (param.printlevel>2)
	printf("\n constrain.  label %.0f, H %i",partt.pos.label,partt.H);

if (param.combin==0) // If not combinatorial search
{
	partt.pos=domain_pos(partt.pos,param); // Take granularity into account	
}

if (param.printlevel>2)
	printf("\n Granularity taken into account");
	
if (param.funct==12) // Graph Coloring problem
{
	for (d=0;d<part.pos.size;d++) // Keep in the box
	{
		if( part.pos.x[d]>param.H.max[d])
		{ 
			partt.pos.x[d]=param.H.max[d];
			continue;
		}

		if( part.pos.x[d]<param.H.min[d])
		{ 
			partt.pos.x[d]=param.H.min[d];
		}
	}

	if (model.H==1) // Coloring/FAP. As few unsatisfied constraints as possible
	{
		xmin=param.H.min[0]; // Min admissible color
		xmax=param.H.max[0]; // Max admissible color
		if (model.confin<4) // Methods 0 to 3
		{
			partt=admissible_color(partt,M,model.confin,cmax); // (Note: matrix M is a global variable)
		}


		if (model.confin==4) // Method 4
		{
			partt=minimize_constraint(partt);
		}
		
		if (model.confin==5) // Method 5
		{
			partt.pos= color_6(M,dplus,part.pos,param.printlevel); // Protein folding
		}
	}
	
	if (model.H==2)  // As few colors as possible
	{
		if (model.confin>0)
		{
			partt=minimize_color(partt,cmax);
		}
	}

	if (model.H==3)
	{
		if (model.confin>0)
		{		
			if (partt.H==1) 
			{
				partt=minimize_constraint(partt);
				//partt=admissible_color(partt,M,model.confin,xmin,xmax,cmax);
			}

			if (partt.H==2) 
			{
				partt=minimize_color(partt,cmax);
			}
			
			partt.H=3-partt.H; // Just for Coloring problem. Switch the projection subspace
		}	
	}

return partt;
}


// Normal PSO	
for (d=0;d<partt.pos.size;d++)
{
	if( partt.pos.x[d]>param.H.max[d])
	{ 
		partt.pos.x[d]=param.H.max[d];
		partt.vel.v[d]=-part.vel.v[d]/2;
		//partt.vel.v[d]=0;
		continue;
	}
		
	if( partt.pos.x[d]<param.H.min[d])
	{ 
		partt.pos.x[d]=param.H.min[d];
		partt.vel.v[d]=-part.vel.v[d]/2;
		//partt.vel.v[d]=0;	
	}
}	
return partt;
}


// ----------------------------------------------------------------------------- DISPLAY_DENSITY
void display_density(int dim,int density_step)
{
// Note: density[][] is a global variable

int 	d;
int 	i;
float sigma;

printf("\n Density matrix. One line / dimension");

for (d=0;d<dim;d++)
{
	printf("\n");
	sigma=0;
	for (i=0;i<density_step;i++)
	{
		printf(" %3i",density[i][d]);
		sigma=sigma+density[i][d];
	}
	printf("  = %.0f",sigma);
}
}


// ----------------------------------------------------------------------------- DISPLAY_MODEL
void display_model(struct model model, int level)
{
printf("\n------\n Model level %i",level);
printf("\n alpha...............:[%1.3f,%1.3f] ",model.alpha_min,model.alpha_max);
printf("\n b ..................:[%1.3f,%1.3f] ",model.b_min,model.b_max);
printf("\n hood_size...........:[%i,%i] ",model.hood_min,model.hood_max);
printf("\n init_size...........: %i ",model.init_size);
printf("\n min_size............: %i ",model.min_size);
printf("\n selection...........: %i ",model.selection);
printf("\n gener_type..........: %i ",model.gener_type);
printf("\n init_type...........: %i ",model.init_type);
printf("\n H ..................: %i ",model.H);
printf("\n confinement ........: %i ",model.confin);
printf("\n parallel ...........: %i ",model.parallel);
printf("\n local_search .......: %i ",model.local_search);
printf("\n rehope .............: %i ",model.rehope);
printf("\n move_type...........: %i ",model.move_type);
printf("\n local queens........: %i ",model.queen);
printf("\n guided_rand.........: %i ",model.guided_rand);
printf("\n rand_type...........: %i ",model.rand_type);

if (model.freedom==1) printf("\n Before a move, all components are modified the same way");
if (model.freedom==2) printf("\n Before a move, each components is modified independently");

}

// ----------------------------------------------------------------------------- DISPLAY_PARAM
void display_param(struct param param)

{
int 	d;
float 	p1,p2,p3;
int 	times;

printf("\n\n PROBLEM");
printf("\n funct...........: %2i",param.funct);
printf("\n Dimension.......: %2i",param.DD);
printf("\n Search space:");
printf("\n xmin    xmax    dx (granul)");
p1=0;p2=0;
p3=-1;// Arbitrary unused value for dx

for (d=0;d<param.H.dim;d++)
	{
	if (param.H.min[d]!=p1 || param.H.max[d]!=p2 || param.H.dx[d]!=p3)
		{
		if (d>0) printf(" (%i times)",times);
		p1=param.H.min[d];
		p2=param.H.max[d];
		p3=param.H.dx[d];
		printf("\n %f  %f  %f",p1 ,p2,p3);
		times=1;
		}
	else
		{
		times=times+1;
		}
	}
printf(" (%i times)",times);
printf("\n target..........: %4.3f",param.target);
printf("\n eps.............: %4.10f",param.eps);
printf("\n printlevel......: %2i",param.printlevel);
printf("\n times...........: %2i",param.times);
printf("\n parapet.........: %f",param.parapet);
if (param.all_diff==1) printf("\nAll components of the solution must be different");
if (param.combin==1)
{
	 printf("\n I do use combinatorial kit");
}

else
{
	printf("\n I don't use combinatorial kit ");
}
printf("\n---------\n");
}

// ----------------------------------------------------------------------------- DISPLAY_POSITION
void display_position(struct position pos)
{
int				d;
int				ncar=0;
struct position post;

post=pos;

printf("\n position size %i\n ",pos.size);
for (d=0;d<pos.size;d++)
{
	printf("%6.6f ",post.x[d]);
	ncar=ncar+10;
	if (ncar>75)
	{
		ncar=0;
		printf("\n");
	}
}
printf (" abs(f-target) = %3.4f",post.f);

}

// ----------------------------------------------------------------------------- DISPLAY_SWARM
void display_swarm(struct swarm sw,int type)
{
int i;

if (type==1)
{
	printf("\n Current swarm at level %i", level);
	for (i=0;i<sw.size;i++)
	{
		display_position(sw.part[i].pos);
		printf("  best: %f",sw.part[i].prev_best.f);
	}
}
if (type==2)
{
	printf("\n Best previous swarm at level %i", level);	
	for (i=0;i<sw.size;i++)
	{
		display_position(sw.part[i].prev_best);
		printf("  new: %f",sw.part[i].pos.f);
	}
}
	
if (type==3)
{
	printf("\n Particle f values");
	for (i=0;i<sw.size;i++)
	{
		printf("\n %i  best: %f new: %f",i,sw.part[i].prev_best.f,sw.part[i].pos.f);
	}
}	

}

// ----------------------------------------------------------------------------- DISPLAY_VELOCITY
void display_velocity(struct velocity vel)
{

int d;

printf("\n velocity. combin = %i",vel.combin);
printf("\n  ");
for (d=0;d<vel.size;d++)
{
	printf("%3.2f ",vel.v[d]);
}


if (vel.combin==1) // "Combinatorial" velocity
{
	printf("\n");
	for (d=0;d<vel.size;d++)
	{
		if (vel.v2[d]==NA)
		{
			printf("    NA");
			continue;
		}
		if (vel.v2[d]==NO)
		{
			printf("    NO");
			continue;
		}
		if (vel.v2[d]==AS)
		{
			printf("    AS");
			continue;
		}
		
		printf("% 3.2f",vel.v[d]);
	}	
}


}

// ----------------------------------------------------------------------------- DISTANCE
float	distance(struct position pos1,struct position pos2)
{
float 	d,dx;
int		i;
d=0;
for (i=0;i<pos1.size;i++)
{
	dx=pos1.x[i]-pos2.x[i];
	d=d+dx*dx;
}
d=(float)sqrt(d);
return d;
}

// ----------------------------------------------------------------------------- DISTANCE_MIN
float	distance_min(struct swarm sw, struct position pos)
{
/*
Let's consider the  set of points S:
- all particle positions

We compute the minimum distance from pos to S

*/
float			dist;
float			dist_min;
int				i;

dist_min=(float)-NA; // Arbitrary too high value

for (i=0;i<sw.size;i++)
{
	dist=distance(pos,sw.part[i].pos);
	if (dist<dist_min) dist_min=dist;
}
return dist_min;
}

// ----------------------------------------------------------------------------- DIVERG
float diverg(struct position pos)
{
// Compute how much the direction of the position is far from the "balanced" one (1,1, ...1)
// (for 2D visualization)


int		d;
float	r;
float	rx;
float 	x;

r=0;
for (d=0;d<pos.size;d++)
{
	r=r+pos.x[d]*pos.x[d];
}	
r=(float)sqrt (r);

rx=(float)pos.size;
rx=(float)(r/sqrt (rx));
x=0;
for (d=0;d<pos.size;d++)
{
	x=x+(pos.x[d]-rx)*(pos.x[d]-rx);
}	
x=(float)sqrt(x);

return x;

}



// ----------------------------------------------------------------------------- DOMAIN_POS
struct position domain_pos(struct position pos,struct param param)
{
// Modify the position according to the "granularity" of the search space
int				d;
struct position post;

post=pos;
for (d=0;d<pos.size;d++)
{
	post.x[d]=regranul(pos.x[d],param.H.dx[d]);
}
	
return post;	
}		

// ----------------------------------------------------------------------------- ENERGY_K
float energy_k(struct particle part,struct search_space H) // Kinetic energy  ( Sum of V^2/2 )
{ // Kinetic energy of a particle
int		d;
float	dx;
float 	e;
float	v;

e=0;

for (d=0;d<part.vel.size;d++)
{
	v=part.vel.v[d];
	dx=H.max[d]-H.min[d];
	if (dx>almostzero)
		e=e+v*v/(dx*dx);
}

return e/2;
}

// ----------------------------------------------------------------------------- ENERGY_K_SWARM
float energy_k_swarm(struct swarm sw) // Kinetic energy  ( Sum of V^2/2 )
{ // Kinetic energy of a swarm
int		i;
float 	e;

e=0;

for (i=0;i<sw.size;i++)
{
	e=e+sw.part[i].vel.ek;
}
return e;
}

// ----------------------------------------------------------------------------- ENERGY_P
float energy_p(struct particle part, float eps)
{ // Potential energy of a particle
float e;
if (eps>almostzero) {e=part.pos.f/eps;} else {e=part.pos.f;};

return e;

}
// ----------------------------------------------------------------------------- ENERGY_P_SWARM
float energy_p_swarm(struct swarm sw)  // Potential energy (Sum of abs(f(X)-target) )
{ // Potential energy of the swarm
int		i;
float 	e;

e=0;

for (i=0;i<sw.size;i++)
{
	e=e+sw.part[i].pos.ep;
}
return e;
}

// ----------------------------------------------------------------------------- ESTIM_EVAL
float estim_eval(struct swarm sw, struct param param,struct model model)
{/* Re-estimate how many evaluations are probably still needed to reach a solution
	globally, for the whole swarm
*/


float	estim;
int		hood;
int		i;
float	nb_estim;

if (param.combin==1) return (float)(1/almostzero); // Combinatorial problem

estim=0;
nb_estim=0;
hood=model.hood_min;

for (i=0;i<sw.size;i++) // Estimate for each particle, and take the max
{
	if(sw.part[i].pos.prev_f[0]<sw.part[i].pos.prev_f[hood-1])
	{
		estim=MAX(estim,estim_eval_particle(sw.part[i],MIN(hood,Max_histo),param.eps));
		nb_estim=nb_estim+1;
	}
}

if (nb_estim==0)
{
	if (param.printlevel>1)
		printf("\n WARNING: impossible to estimate the number of evaluations needed");
	estim=(float)(1/almostzero);
}
else
{
	estim=sw.size*(estim+hood);// Add cycle_size and multiply by the number of particles

	estim=10*estim; // Arbitrary pessimistic point of view
	estim=MIN(estim,(float)(1+1/almostzero));
}

if (model.move_type==2) // Each move needs two evaluations
{
	estim=2*estim;
}

if (model.local_search==1)
{
	estim=estim+param.DD;
}

if (model.local_search>1)
{
	estim=estim+param.DD*param.DD;
}

if (param.funct==12) // Coloring problem is more difficult ...
{
	estim=estim*dplus.dmax;
}
return estim; 
}

// ----------------------------------------------------------------------------- ESTIM_EVAL2
float estim_eval2(struct swarm sw, struct param param,struct model model, int cycle_size)
{/* Re-estimate how many evaluations are probably still needed to reach a solution
	globally, for the whole swarm

Note: hyper_vol[] is a static table, containing unit hypersphere volumes for dimension 1,2,...300
Note: level and volume_checked[level] are global variables
*/
int				d;
float			df;
float			df_mean;
float			dim;
float			dmax;
float			ds;
float			dx;
float			dx_mean;
float			estim;
int				i;
int				j;
int				k;
struct subswarm	hood;
int				ncount;
float			r;
float			search_power_hood;
float			search_power_swarm;
float			swarm_volume;

if (param.combin==1) return (float)(1/almostzero); // Combinatorial problem. The following heuristic does'nt work

dim=(float) param.H.dim;
search_power_swarm=0;

/*
swarm_volume=1;
for (d=0;d<dim;d++) // Volume of the "best" swarm
{
	dmax=0;
	for (i=0;i<sw.size;i++) // For each particle ...
	{
		for (j=0;j<i;j++)
		{
			dx=fabs(sw.part[i].prev_best.x[d]-sw.part[j].prev_best.x[d]);
			if (dx>dmax) dmax=dx;
		}
	}
	swarm_volume=swarm_volume*dx;
}
*/

for (i=0;i<sw.size;i++) // For each particle ...
{
	hood=hood_swarm(sw,i,param.eps); // ... find the neighbourhood

	dx_mean=0;
	df_mean=0;
	ncount=0;
	for (j=0;j<hood.size;j++) // For each neighbour (including the particle itself ...
	{
		k=hood.rank[j];
		dx=distance (sw.part[i].pos,sw.part[k].prev_best);
		if (dx<almostzero) continue;
			df=fabs (sw.part[i].pos.f-sw.part[k].prev_best.f);
			dx_mean=dx_mean+dx;
			df_mean=df_mean+df/dx;	
			ncount=ncount+1;
	}	
	df=df_mean/(float)ncount;
	
	r=param.eps/df;
		
	if (dim>1)
	{
		ds=hyper_vol[(int)(dim-2)]*pow(r,dim-1);// small hypersphere
	}
	else
	{
		ds= 1;			
	}
			
	// "volume" of the cylinder which could be checked
	// (Note: this formula assumes that alpha is constant)														
	search_power_hood=(model.b_max-model.b_min)*ds*dx_mean;  
	
	search_power_swarm=search_power_swarm+search_power_hood;
}

if (search_power_swarm<almostzero) return 1/almostzero;
printf("\n volume_checked[level] %f, search_power_swarm %f, H.volume %f",volume_checked[level],search_power_swarm,param.H.volume);

volume_checked[level]=volume_checked[level]+search_power_swarm*(float)cycle_size;
estim=((param.H.volume-volume_checked[level])/search_power_swarm)*(float)sw.size; // Estimation of the number of evaluations

return estim; 
}

// ----------------------------------------------------------------------------- ESTIM_EVAL3
float estim_eval3(int nb_past_eval,float eps)
{
 /* Re-estimate how many evaluations are probably still to do
    We consider the points (u,v) := (nb_of_eval, best_f_value)
	and approximate the curve with v=lambda/u (option: lambda/u^2)  Then, we solve v<eps. 
Note: best_eval[][2] is a global variable
Note: level is a global variable
 */

int		i;
double	delta;
double	lambda;
double	mu;
int		nb;
int		option;
double 	s1,s2;
double	u;
double	u0;
double	u1;
double	uc;
double	v1;
double	vc;
option=1;

// model lambda/(u-u0)^mu
nb=nb_past_eval;
if (nb>Max_histo-1) nb=Max_histo-1;
s1=0;s2=0;
u0=best_eval[nb][0][level];
u1=best_eval[nb-1][0][level];
v1=best_eval[nb-1][1][level];
uc=best_eval[0][0][level];
vc=best_eval[0][1][level];

if (v1<almostzero) return 1/almostzero;
if (v1<=vc) return 1/almostzero;
if (u1<=u0) return 1/almostzero;

mu=log(vc/v1)/log((u1-u0)/(uc-u0));
lambda=pow(u1-u0,mu)*v1;

//printf("\n log(vc/v1) %f",log(vc/v1));
//printf(" log(...) %f",log((u1-u0)/(uc-u0)));
//printf("\n v1,vc: %f %f.  lambda, mu: %f %f",v1,vc,lambda,mu);

u=pow(2*lambda/eps,1/mu) + u0-uc;
return u;

/*
for (i=0;i<nb;i++)
{
	u=best_eval[i][0][level]-u0;
//printf("\n u,v: %f %f",u, best_eval[i][1][level]);
	if (u<almostzero) return 1/almostzero;
	
	if (option==2)
	{
		s1=s1+best_eval[i][1][level]/(u*u);
		s2=s2+1/(u*u*u*u);
	}
	else
	{
		s1=s1+best_eval[i][1][level]/(u);
		s2=s2+1/(u*u);
	}
	
}
//if (s2<almostzero) return 1/almostzero;

lambda=s1/s2; 

//printf("\n nb %i, s1, s2, %f %f, lambda %f, u0, u %f %f",nb,s1,s2,lambda,u0,uc);
//printf("\n");
//for (i=0;i<=nb;i++) printf(" %.0f",best_eval[i][0][level]);

if (option==2)
{
	delta=vc/(lambda*(uc-u0)*(uc-u0));
	 u=sqrt(lambda*delta/eps);
	} 
else 
{
	delta=vc/(lambda*(uc-u0));
	u=lambda*delta/eps;
}

u=u+u0-uc;

return u;
*/

}


// ----------------------------------------------------------------------------- ESTIM_EVAL_PARTICLE
float estim_eval_particle(struct particle part,int nb_past_eval, float eps)
{
 /* Re-estimate how many evaluations are probably still to do
    using previous evaluations for a particle which have improved its position
	For this particle, we consider the points (u,v) := (evaluation rank, f value)
	and approximate the curve with v=lambda/u. Then, we solve v<eps. 
 */


int		i;
float 	estim;
float 	s1,s2;
float	x;

s1=0;s2=0;
for (i=0;i<nb_past_eval;i++)
{
	x=(float)(nb_past_eval-i);
	s1=s1+part.pos.prev_f[i]/x;
	s2=s2+1/(x*x);
}
estim=s1/s2; 

return estim/eps;
}

// ----------------------------------------------------------------------------- EVAL
float eval(float total,float target)
{ // How far from the target is the value "total"
float v;
v=(float)fabs(target-total);
return v;
}

// ----------------------------------------------------------------------------- HOMOGEN_TO_CARTE
struct position	homogen_to_carte(struct position pos)
{
/* Convert a position given by its homogeneous coordinates into 
the same position given by its cartesian coordinates. The homogeneous coordinates
are supposed to be given relatively to the unit points:
(0, ... 0), (1,0,...0) (0,1,0...,0) ... (0,...,0,1)

See, for example, Apple Trees problem

*/
int				d;
struct position post;
float			s;

//post=pos;
post.size=pos.size-1;

s=0;
for (d=0;d<pos.size;d++)
{
		s=s+pos.x[d];
	}
if (fabs (s)<almostzero)
{
	printf("\n WARNING. Incorrect homogeneous coordinates. Return null point");
	for (d=0;d<post.size;d++)
		post.x[d]=0;
	return post;
}

for (d=0;d<post.size;d++)
	post.x[d]=pos.x[d+1]/s;

return post;
}

// ----------------------------------------------------------------------------- HOOD_QUEEN
struct particle	hood_queen(struct swarm sw,struct subswarm hood_s,struct param param,struct model model)
{
/*
 Weighted position (queen) of the particles in a given neighbourhood,
 using a second order approximation of the objective function
*/
float			c;
float			c_tot;
int				d;
int				dim;
int				i;
struct particle queen={0};
int				rank;

dim=sw.part[0].pos.size;

c_tot=0; 
for (i=0;i<hood_s.size;i++) // Total weight
{
	rank=hood_s.rank[i];
	c_tot=(float)(c_tot+1/sqrt (sw.part[rank].pos.f));
}
	
for (d=0;d<dim;d++)// Gravity center
{	
	queen.pos.x[d]=0;
	queen.vel.v[d]=0;
	
	for (i=0;i<hood_s.size;i++) 
	{
		rank=hood_s.rank[i];
		c=(float)(1/sqrt (sw.part[rank].pos.f));
		queen.pos.x[d]=queen.pos.x[d] + sw.part[rank].pos.x[d]*c;
		queen.vel.v[d]=queen.vel.v[d] + sw.part[rank].vel.v[d]*c ;
	}
	queen.pos.x[d]=queen.pos.x[d]/c_tot;
	queen.vel.v[d]=queen.vel.v[d]/c_tot;
}

queen=complete_part(queen, param, model);

return queen;
}

// ----------------------------------------------------------------------------- HOOD_SWARM
struct subswarm hood_swarm(struct swarm sw,int i,float eps)
{ // Define the neighbourhood of the particle i. Contains i itself
  // Note that it is here a "social" neighbourhood, not a "physical" one
int				h2;
struct subswarm	hood_s;
int				hood_size;
int				j;
int				k;
int 			N;
int				rank;

N=sw.size;
hood_size=(int)sw.part[i].hood_size;
//fprintf (f_run,"\npart %i hood_size %f",i,sw.part[i].hood_size);

h2=(int)(0.5*MIN((float)N,(float)hood_size));

hood_s.size=0;

for( k = -h2;k<=h2;k++)     //defines neighbourhoods
{
	rank = i + k ;             //neighbourhood is i + (-hood_size/2,0,+hood_size/2)
	// Neighbourhood is "circular"
	if( rank < 0)
	{ 
		rank = N + rank;
	}
	else
	{
		if( rank >= N)
		{
			rank = rank - N;
		}
	}
	hood_s.rank[hood_s.size]=rank;
	hood_s.size=hood_s.size+1;								
}
	
hood_s.best=0;
hood_s.worst=0;

for (j=1;j<hood_s.size;j++) // In the neighbourhood of i, find the best and the worst
{
	rank=hood_s.rank[j];
	//if (better_than(sw.part[rank],sw.part[hood_s.rank[hood_s.best]],eps)>1) hood_s.best=j;
	if (better_than(sw.part[rank],sw.part[hood_s.rank[hood_s.best]],eps)>0) hood_s.best=j;
	if (better_than(sw.part[hood_s.rank[hood_s.worst]],sw.part[rank],eps)>0) hood_s.worst=j;
}
	
return hood_s;
}

// ----------------------------------------------------------------------------- IMPROVEMENT
float	improvement(struct position pos)
{ // self improvement of a given particle
float improv;

improv=(pos.prev_f[1]-pos.prev_f[0])/(pos.prev_f[1]+pos.prev_f[0]);

// Note: remember that, in fact, f is equal to abs(objective_function - target)

return improv;
}

// ----------------------------------------------------------------------------- INIT_PARTICLE
struct particle	init_particle(struct param param, int type, struct model model,struct swarm sw,int rank_model)
{ // Initialisation of a particle
int				d;
float			dist_min;
float			dx;
int				loop;
int				loop_max;
int				max,min;
int				option;
struct particle part={0};
float			rate; // "mutation" rate
int				alea_type;
float			xconv;
float			xd;
float			xmax,xmin;
float			vd;

option=model.guided_rand; // If >0, use alea_density() Guided Random Generation

part.pos.size=param.DD;
dist_min=0;
loop=0;
loop_max=param.DD;

if (type<0) goto special; // Generate a special particle

if (param.combin==0 && param.funct!=12) // Normal PSO (don't use Combinatorial kit)
{
	if (type==1) // Add it according to the "model"
	{	
		if (alea_float(0,1)<0.5 && fabs(sw.part[rank_model].pos.prev_f[1]-sw.part[rank_model].pos.prev_f[0])>0 )
		{
			rate=sw.part[rank_model].pos.prev_f[0]/(sw.part[rank_model].pos.prev_f[1]-sw.part[rank_model].pos.prev_f[0]);
			for( d = 0;d<param.DD;d++)  //for each dimension
			{
				xd=sw.part[rank_model].pos.x[d];
				vd=sw.part[rank_model].vel.v[d];
				dx=vd*rate;
				if( dx>0) 
				{
					xmin=xd;
					xmax=MIN(xd+2*dx,param.H.max[d]);
				}
				else
				{
					xmin=MAX(xd+2*dx,param.H.min[d]);
					xmax=xd;
				}
				part.pos.x[d] = alea_float(xmin,xmax);   //x(i,d) in [xmin, xmax]
			}
		}
		else  // Just random generation
		{
			if (option==0)
			{
				rate=alea_float(0,1);	
				for( d = 0;d<param.DD;d++)  //for each dimension
				{
					xd=sw.part[rank_model].pos.x[d];

					dx=rate*(param.H.max[d]-param.H.min[d]);
					xmin=MAX(xd-dx,param.H.min[d]);
					xmax=MIN(xd+dx,param.H.max[d]);
					part.pos.x[d] = alea_float(xmin,xmax);   //x(i,d) in [xmin, xmax
				}
			}	
			if (option>0) // Guided Random Generation
			{
				alea_type=alea(0,1);
//alea_type=1;
				for( d = 0;d<param.DD;d++)  //for each dimension
				{
					part.pos.x[d]=alea_density(density_step, d, param.H.min[d],param.H.max[d],alea_type);
				}
				goto velocity;
			}
		}
	}

	if (type==0) // Completely new particle
	{	
		if (option==0)
		{
			for( d = 0;d<param.DD;d++)  //for each dimension
			{
				part.pos.x[d] = alea_float(param.H.min[d],param.H.max[d]);   //x(i,d) in [xmin, xmax]
			}
		}
		else // Guided Random Generation
		{
			alea_type=alea(0,1);
//alea_type=1;
			for( d = 0;d<param.DD;d++)  //for each dimension
			{
				part.pos.x[d]=alea_density(density_step, d, param.H.min[d],param.H.max[d],alea_type);
			}
		}
	}
}

velocity:

xconv=(float)(density_step-1);

for( d = 0;d<param.DD;d++)  //for each dimension
{
	//vmax=(param.H.max[d]-param.H.min[d])/(1+pow(param.N,1/param.DD));
	//vmax=(param.H.max[d]-param.H.min[d])/xconv;
	//part.vel.v[d] = alea_float(-vmax,vmax);
	part.vel.v[d] = alea_float(part.pos.x[d]-param.H.min[d],param.H.max[d]-part.pos.x[d]);
}

//-------------------------------	
if (param.combin==1 && param.funct!=12) // Use Combinatorial kit
{
	for( d = 0;d<param.DD;d++)  //for each dimension
	{
		// WARNING. We DO suppose coding values are al integers of [min,max]
		min=(int)param.H.min[d];
		max=(int)param.H.max[d];
		part.pos.x[d] = (float)alea(min,max);
		part.vel.v2[d]=NA; // random velocity	
	}
	
	part.vel.ek=0;	// No meaning ...
}

special:

	
//--------------------------------
if (param.funct==12)
{	
	part= init_particle_color( param, model,  M,  dplus,type);
}
//--------------------------------

part=complete_part(part,param, model);

if (param.printlevel>1) printf("\n init => %f",part.pos.f);	

return part;
}

// ----------------------------------------------------------------------------- INIT_SWARM
struct swarm init_swarm(struct param param, struct model model)
{ // Random initialisation of the swarm
int				d;
int 			i;
int				p_size;
int				status;
struct swarm	sw;

if (param.printlevel>0) 
	printf("\n Level %i. INIT swarm", level);


for (d=0;d<param.DD;d++) // Initialize density of checked positions
{
	for (i=0;i<density_step;i++)
	{
		density[i][d]=0;
	}
}

sw.size=0;	

// Special case Coloring/FAP. A "null" particle is added, and a "special" one

if (param.funct==12) 
{
	if (param.printlevel>2) printf("\n init special particle 0");	

	sw.part[0]=init_particle(param,-1,model,sw,0);
	sw.size=1;
	if(chance[level]==1 ) goto end_by_chance;

	if (param.printlevel>2) printf("\n init special particle 1");	
	sw.part[1]=init_particle(param,-2,model,sw,0);
	sw.size=2;

	if(chance[level]==1 ) goto end_by_chance;
}
	
// Initial swarm read on a file
	
if (param.init_file>0) 
{
	
	f_init_r=fopen(param.init_swarm_r,"r");
	printf("\n Initial swarm read on %s",param.init_swarm_r);
	
	fscanf (f_init_r,"%i",&p_size);
	if (p_size!=param.DD)
	{
		printf("\n WARNING");
		printf("\n Not consistent init file. DImension is %i, and data are for dimension %i",param.DD,p_size);
		printf("\n I generate an initial swarm");
		goto generate;
	}
	
	
	read_particle:
//printf("\n read particle %i",sw.size+1);
	fscanf (f_init_r,"%i",&status);
	if (status==-1) goto f_close;	
		if (status==1) // This position was quite good. Take it as initial particle
		{
		
			for (d=0;d<param.DD;d++) 
			{
				fscanf(f_init_r,"%f", &sw.part[sw.size].pos.x[d]);
				sw.part[sw.size].vel.v[d] = 0;
			}
			sw.part[sw.size].pos.size=param.DD;	
			sw.part[sw.size].vel.size=param.DD;
			sw.part[sw.size].vel.ek=0; // Kinetic energy
			sw.part[sw.size]=complete_part(sw.part[sw.size],param,model);
			sw.size=sw.size+1;
		}

		if (sw.size<param.N) goto read_particle;
		
	
	f_close:
	fclose (f_init_r);
	
	
	if (sw.size<param.N) // Eventually, complete by generated particles
	{
		for( i = sw.size;i<param.N;i++) // for each individual
		{
			if (param.printlevel>2) printf("\n generate particle %i",i+1);
			sw.part[i]=init_particle(param,0, model,sw,0);
			if(chance[level]==1 ) goto end_by_chance;
		}
	}
	sw.size=param.N;
}	
	
else //Complete generation of particles
{
	generate:		
	
	for( i = sw.size;i<param.N;i++) // for each individual
	{
//printf("\n    init particle %i, size %i", 1+i, sw.size);
		sw.part[i]=init_particle(param,0, model,sw,0);
		if(chance[level]==1 ) goto end_by_chance;
		
	}
	sw.size=param.N;
}
//------------
if (param.printlevel>0) 
	printf(", size %i", sw.size);
	
sw.best=best(sw); // Find the best

sw.worst=worst(sw); // find the worst			

if(param.printlevel>0)
	printf("\n--------------------\n Best |f-target| at initialisation: %f",	sw.part[sw.best].pos.f);
	

if (param.printlevel>1)
{
	printf(" display swarm, size %i",sw.size);
	display_swarm(sw,1);
}

E_p=energy_p_swarm(sw);
E_k=energy_k_swarm(sw);

E0=E_p+E_k;
if(param.printlevel>0)
	printf("\n Potential energy %f,  Kinetic energy %f",E_p,E_k);

return sw;


end_by_chance:
if(param.printlevel>0)
	{ 
		printf("\n Chance at initialization");
	}
return sw;
}

// ----------------------------------------------------------------------------- LIPSCHITZ
float lipschitz(struct swarm sw, int part_rank, struct subswarm hood)
{
/* Estimate the mean Lipschitzian coefficient for the error function f "around" a particle
*/
float			df;
float			dist;
int				i;
int				j;
float			lambda;
float			ncount;

lambda=0;
ncount=0;
for (i=0;i<hood.size;i++)
{
	j=hood.rank[i];
	dist=distance(sw.part[part_rank].pos,sw.part[j].prev_best);
	if (dist<almostzero) continue;
		df=fabs (sw.part[part_rank].pos.f-sw.part[j].prev_best.f)/dist;
		lambda=lambda + df;
		ncount=ncount+1;
}
return lambda/ncount;
}

// ----------------------------------------------------------------------------- MAX 
float MAX(float a,float b)
{
if (a>b) return a; return b;
}


// ----------------------------------------------------------------------------- MAX_COMP
float max_comp(struct position pos)
{// Find the max ot the components of a position
int		d;
float 	max;

max=pos.x[0];
for (d=1;d<pos.size;d++)
	if (pos.x[d]>max) max=pos.x[d];

return max;
}

// ----------------------------------------------------------------------------- MIN 
float MIN(float a,float b)
{
if (a<b) return a; return b;
}

// ----------------------------------------------------------------------------- MIN_COMP
float min_comp(struct position pos)
{// Find the min ot the components of a position
// Global variable NA is a big <0 number. Means "non assigned"
int		d;
float 	min;

min=(float)-NA;
for (d=0;d<pos.size;d++)
{
	if (pos.x[d]<=NA) continue;
		if (pos.x[d]<min) min=pos.x[d];
}

return min;
}

// ----------------------------------------------------------------------------- MOVE_PARTICLE
struct particle	move_particle(struct particle part,struct particle partg,struct param param, struct model model,int rank)
{ //  ********* CORE OF THE ALGORITHM **********************************************************


float			alpha;
float			b;
float			beta;
int				cmax;
int				d;
int				d_max,d_min;
int				DD;
int				freedom;
float			kappa;
float			ksi;
float			m,ma,mb;
float			ma_, mb_;
int				option;
//float			partial;
struct particle partt;
struct particle partt0;
float			pgd;
float			phi;
float			pid;
float			total;
struct velocity	v1,v2;
float			vid;
float			xid;

move[0]=move[0]+1;
//option=1; kappa=(float)0.8; // Use complete PSO Type 1"
option=0; // Use simplified PSO

//partial=50; // % of components to modify 

partt=part;
partt0=part;
DD=part.pos.size;

if (param.printlevel>2)
{
	printf("\n move_particle, x and pg");
	display_position(part.pos);
	display_position(partg.prev_best);
}

freedom=model.freedom;
if (rank<0)
{
	d_min=0;d_max=DD;
}
else
{
	d_min=rank;d_max=rank;
}	

if (option==0)
{
	/*
	ma=improvement(part.pos);
	mb=improvement(partg.pos);
	m=(ma-mb)/2;
	alpha=adapt_alpha(m,model);
	m=(partg.pos.f-part.pos.f)/(partg.pos.f+part.pos.f);
	b=adapt_b(m,alpha,model);
	*/
	
	ma=particle_improv(part);
	mb=particle_improv(partg);
	ma_=fabs(ma);
	mb_=fabs(mb);
	if (ma_+mb_<almostzero) m=0; else m=(ma-mb)/(ma_+mb_);
	
	alpha=adapt_alpha(m,model);
	b=adapt_b(-m,alpha,model);

//if (b<0) 
//printf("\n move_particle. alpha %f b %f, m %f",alpha,b,m);	

	switch (freedom)
	{
		case 0:
			beta=b;
			break;
		case 1:
			beta=alea_float(model.b_min,b);
			break;
		case 2:
			beta=alea_float(0,b);
			break;
		case 5:
			beta=alea_float((float)(0.8*b),(float)(1.2*b));
		default:
			beta=b;
			break;
	}	
}

if (option==1)
{
	phi=(model.alpha_max+model.b_max)/2;
	if (phi==4) phi=(float)4.1;
	if (phi>4)
	{
		ksi=(float)(2*kappa/fabs((phi-2+sqrt(phi*(phi-4)))));
	}
	else
	{
		ksi=kappa;
	}
}

move:

if (param.combin==0) // Normal PSO (don't use combinatorial kit
{
	if (param.funct!=12) // Normal case
	{
		for ( d = d_min;d<d_max;d++) // for each dimension // HERE THE PARTICLE MOVES *****
		{
			//if (alea_float(0,1)<1-partial/100) continue;
			
			pgd=partg.prev_best.x[d];
			vid=part.vel.v[d]; // Define a couple ot temp. variables
			xid=part.pos.x[d];
	
			if (option==0)
			{
				// Optionnaly, you may change beta for each dimension
				if (freedom==3) 
				{
					beta=alea_float(model.b_min,b);			
				}
				if (freedom==4) 
				{
					beta=alea_float(0,b);
				}


				/* *******************************************  HERE IS THE BASIC MOVE  */		
				partt.vel.v[d]  = alpha*vid  + beta*(pgd-xid) ;                     /* */
				partt.pos.x[d]  = xid + partt.vel.v[d];                           /* */
				/* ******************************************************************  */
			
			}
			
			if (option==1)
			{
				alpha=alea_float(model.alpha_min,model.alpha_max)/2; // phi1
				beta=alea_float(model.b_min,model.b_max)/2; // phi2
				
				pid=part.prev_best.x[d];
				partt.vel.v[d]  = ksi*(vid  + alpha*(pid-xid) + beta*(pgd-xid)) ;
				partt.pos.x[d]  = xid + partt.vel.v[d]; 
			
			}
		}	
		
		if (model.move_type==2)
		{	
			// Try the self-move (not using the best neighbour)
			self_better[0]=self_better[0]+1;// Just for stat
			
			for (d=d_min;d<d_max;d++)
			{
				partt0.pos.x[d]  = partt0.pos.x[d] + partt0.vel.v[d];
			}
			total=tot(partt0.pos, param, model);       
			partt0.pos.f=eval(total,param.target);
			
			if (partt0.pos.f<partt.pos.f) 
			{
//printf("\n self-move is better %f < %f",partt0.pos.f,partt.pos.f);
				partt=partt0;
				self_better[1]=self_better[1]+1;// Just for stat
			}
		}	
			
	}
	
	if (param.funct==12 && param.combin!=1) // Special case Coloring/FAP
	{
		for (d=d_min;d<d_max;d++) v2.v[d]=partg.prev_best.x[d]-part.pos.x[d];

		v2.size=part.vel.size;
		//partt.vel=update_vel_color(part.pos,part.vel,v2,alpha,beta);

		cmax=(int)param.max_fr;
		partt.vel=update_vel_color2(part.pos,part.vel,v2,alpha,beta,cmax);
	
		for (d=d_min;d<d_max;d++)
		{
			partt.pos.x[d]  = part.pos.x[d]+ partt.vel.v[d]; // WARNING: position may be <0
		}
	}			
}
	
if (param.combin==1) // Use combinatorial kit
{
	v1=pos_minus_pos(partg.prev_best,part.pos,param.printlevel);
	v1.combin=1;
	
	/*
	v1=coeff_times_vel(v1,beta/(alpha+beta),param);
	v1=vel_granul(v1,param.H);
	v2=coeff_times_vel(part.vel,alpha/(alpha+beta),param);
	v2=vel_granul(v2,param.H);
	partt.vel=vel_plus_vel(v1,v2,param);
	partt.vel=vel_granul(partt.vel,param.H);
	*/
	partt.vel=coeff_times_vel2(part.vel,v1,alpha,beta,rank);
	
	if (param.printlevel>2)
	{
		printf("\n move_particle, velocity");
		display_velocity (partt.vel);
	}	
	partt.pos=pos_plus_vel(partt.pos,partt.vel,param);
	
	for (d=0;d<DD;d++) // Randomize the velocity
	{
		if (partt.vel.v2[d]!=NO) partt.vel.v2[d]=NA;
	}
	
	partt=auto_move(partt, param,model);
}

// Adjustements -------	
if (param.printlevel>2)
{
	printf("\n  Adjustements");
	display_position(partt.pos);
}

partt=constrain(partt,param,model); // Keep it in the search space


if (param.funct==12 && model.local_search>0 && param.combin==0) // For Coloring/FAP
{
	partt.pos=color_local_search(partt.pos,param,model);
}

if (param.all_diff>0) // Eventually, modify so that all components are different
	partt.pos=all_different(partt.pos,param);
	
if (param.combin==0) // As the new position may have changed, the velocity has to be adjusted 
{
	for (d=0;d<partt.vel.size;d++)
	{
		partt.vel.v[d]=partt.pos.x[d]-part.pos.x[d];
	}
}
	
if (param.combin==1)
{
	partt.vel=pos_minus_pos(partt.pos,part.pos,0);
}
	
partt.vel.combin=param.combin; // Memorize the kind of velocity (combinatorial or not)
// End od adjustements ------		
	
total=tot(partt.pos, param, model);       // ********** evaluation of the position
partt.pos.f=eval(total,param.target);

partt.pos.X1=X1; // For Leonardo visualization
partt.pos.X2=X2;	
	

// Shift prev_f
for (d=0;d<Max_histo-1;d++)
{
	partt.pos.prev_f[Max_histo-d-1]=part.pos.prev_f[Max_histo-d-2];
}
partt.pos.prev_f[0]=partt.pos.f; // Save position value as "previous position"	

partt.prev_prev_best_f=partt.prev_best.f;
	
if( partt.pos.f < partt.prev_best.f) // if it's better than previous best so far
{  
	partt.prev_best = partt.pos;  // save best position
}
	
	
if (partt.pos.f <best_result[level].f)
	move[1]=move[1]+1; // Just for stat	
	
// Update energies
partt.pos.ep=energy_p(partt,param.eps);

if( param.combin==0) // Normal PSO (Don't use Combinatorial kit)
{
	partt.vel.ek=energy_k(partt,param.H);
}
if (param.combin==1) // Use Combinatorial kit
{
	partt.vel.ek=(float)sqrt(partt.vel.size);
}

if(partt.pos.f<param.eps)  // You may find a solution by chance
{
	best_result[level]=partt.pos;
	chance[level]=1;
}
if (param.printlevel>2) display_position(partt.pos);


partt.pos.X1=X1; // For Leanordo visualization
partt.pos.X2=X2;
		
return partt;
}

// ----------------------------------------------------------------------------- MOVE_SWARM
struct swarm	move_swarm(struct swarm sw,struct param param, struct model model,int rank)
{
int				g;
struct subswarm	hood;
int				i;
struct swarm	swt;


swt=sw;

for( i = 0;i<swt.size;i++)          // For each particle ...
{	 
	if (model.parallel== 1) // ... find neighborhood best (parallel) ...
	{
		hood=hood_swarm(sw,i,param.eps);
		g=hood.rank[hood.best];
	}
	else //  ... find neighborhood best (sequential)...
	{
		hood=hood_swarm(swt,i,param.eps);
		g=hood.rank[hood.best];
	}
	
	E_p=E_p-swt.part[i].pos.ep; // Remove old energy values
	E_k=E_k-swt.part[i].vel.ek;
	

	swt.part[i]=move_particle(swt.part[i],swt.part[g],param, model,rank); // ... move ...
	
	E_k=E_k+swt.part[i].vel.ek; // Add new energy values
	E_p=E_p+swt.part[i].pos.ep;
	
	if (chance[level]==1) return swt;
}

swt.best=best(swt);// ...rank of population PREVIOUS best

return swt;
}

// ----------------------------------------------------------------------------- PARTICLE_IMPROV
float particle_improv(struct particle part)
{
float improv;

//improv=(part.prev_prev_best_f-part.prev_best.f)/part.prev_prev_best_f;
improv=(part.pos.prev_f[1]-part.pos.prev_f[0])/(part.pos.prev_f[1]+part.pos.prev_f[0]);
return improv;

}
// ----------------------------------------------------------------------------- POS_TO_MODEL
struct model pos_to_model(struct position pos, struct model model)
{
/* Transform a position (level 0) into a model for level 1
(Note: the paramater model is here just for test purpose)

Useful for recursive PSO
*/

struct model	modelt;

modelt=model;

modelt.alpha_max=pos.x[0];
modelt.alpha_min=pos.x[1];
modelt.b_max=pos.x[2];
modelt.b_min=pos.x[3];
modelt.hood_min=(int)pos.x[4];
modelt.hood_max=(int)pos.x[5];
modelt.max_size=(int)pos.x[6];

return modelt;
}

// ----------------------------------------------------------------------------- PRINT_N_RUN
void print_N_run(struct position best_result,struct param param)
{
float 		diff;
float		S;
float 		z,z1;
z=eval_f_tot[level]; // Evaluations here has not to be taken into account
z1=eval_f[level];

// Some stuff to print at the end

S=param.target;
diff=best_result.f;
printf("\n");

if(param.printlevel>1)
{ 
	display_param(param);
}
	
printf( "\nResult.................: %4.10f",best_result.f) ;  //show us how close it came
printf("  (wanted < %4.10f)",param.eps);
printf(" by part. %.0f",best_result.label);
if (fabs(S)>almostzero) printf( "\nRelative diff..........: %4.4f",fabs(diff/S));
	
if (param.printlevel>0) display_position(best_result);
	
printf("\nTotal number of evaluations: %10.0f",z);
printf("\nTotal number of clock ticks: %ld",clock_tick-1);

eval_f_tot[level]=z;
eval_f[level]=z1;
				
}

// ----------------------------------------------------------------------------- READ_MODEL
struct model read_model(FILE *f_models)
{
float			bsup;
struct model 	model;

fscanf (f_models, "%f",&model.alpha_max);
fscanf (f_models, "%f",&model.alpha_min);
fscanf (f_models, "%f",&model.b_max);
fscanf (f_models, "%f",&model.b_min);
fscanf (f_models, "%i",&model.hood_min);
fscanf (f_models, "%i",&model.hood_max);

fscanf (f_models, "%i",&model.init_size);
fscanf (f_models, "%i",&model.min_size);
fscanf (f_models, "%i",&model.max_size);
fscanf (f_models, "%i",&model.selection);

fscanf (f_models, "%i",&model.freedom);

fscanf (f_models, "%i",&model.gener_type);
fscanf (f_models, "%i",&model.init_type);
fscanf (f_models, "%i",&model.H);
fscanf (f_models, "%i",&model.confin);
fscanf (f_models, "%i",&model.parallel);
fscanf (f_models, "%i",&model.local_search);
fscanf (f_models, "%i",&model.rehope);
fscanf (f_models, "%i",&model.move_type);
fscanf (f_models, "%i",&model.queen);
fscanf (f_models, "%i",&model.guided_rand);
fscanf (f_models, "%i",&model.rand_type);

if(model.hood_max>Max_hood)
{
	printf("\n *** WARNING ***, hood size too big for me (see Max_hood in source code)");
	printf("\n  I take only %i",Max_hood);
	model.hood_max=Max_hood;
}

if(model.max_size>Max_swarm_size)
{
	printf("\n *** WARNING ***, Max swarm size too big for me (see Max_swarm_size in source code)");
	printf("\n  I take only %i",Max_swarm_size);
	model.max_size=Max_swarm_size;
}
	
bsup=(float)(model.alpha_max+1+2*sqrt (model.alpha_max));
if (model.b_min>bsup)
{
	printf("\n WARNING. You should either increase alpha_max or decrease b_min (or both)");
	printf("\n          for sure convergence (you should have b<alpha+1+2*sqrt(alpha) )");
}
	
return model;
}
// ----------------------------------------------------------------------------- READ_PARAM
struct param read_param(FILE *f_param, FILE *f_data,int begin)
{
int				d,d_current,d_times;
struct param 	param;

printf("\n-----------\n Read param");

strcpy(param.data,"None"); // Default. No special data file
param.init_file=0; // Default value: no data file for initial swarm. 1 else
strcpy(param.init_swarm_r,"None"); // Default value. Initial swarm NOT read from a file
param.mine=0; // Default. Non user defined function
param.used_constr=1; // Default value: use all constraints. <1 else (cf Frequency Assignment problem)
param.combin=0; // Default value: don't use combinatorial kit. 1 else

	//--------------------------------- Read parameter file
	if (begin==0)
		fscanf(f_param,"%i",&param.batch_runs);

	printf("\n\n ***********************************************************************");

	fscanf (f_param,"%i",&param.funct);
	if (param.funct==my_function) fscanf(f_param,"%i",&param.mine);
	fscanf (f_param,"%i",&param.DD);
	
	
	// Search space
	fscanf (f_param,"%i",&param.combin);
	param.H.dim=param.DD;
	
	d_current=0;
	new_d:
	fscanf (f_param,"%i",&d_times); // Number of times you define the same interval and granularity
	fscanf (f_param,"%f",&param.H.min[d_current]);
	fscanf (f_param,"%f",&param.H.max[d_current]);
	fscanf (f_param,"%f",&param.H.dx[d_current]);
	d_current=d_current+1;
	if (d_times>1)
	{
		for (d=d_current;d<d_current+d_times-1;d++)
		{
			param.H.min[d]=param.H.min[d-1];
			param.H.max[d]=param.H.max[d-1];
			param.H.dx[d]=param.H.dx[d-1];
		}
		d_current=d_current+d_times-1;
	}
	
	if (d_current<param.DD) goto new_d;
		
		
	fscanf (f_param,"%f",&param.target);
	fscanf (f_param,"%f",&param.eps);

	fscanf (f_param,"%i",&param.all_diff);

	fscanf (f_param,"%i",&param.printlevel);
	fscanf (f_param,"%i",&param.save);
	fscanf (f_param,"%i",&param.times);



if (param.funct==12) // Frequency Assignment problem
{
	fscanf (f_param,"%s",&param.data);
	printf("\n Data file %s",param.data);
	f_data=fopen(param.data,"r");
	fscanf (f_param,"%f",&param.used_constr); // Ratio of constraints really used
	fscanf (f_param,"%f",&param.max_fr); // Max number of colors wanted
	
	 M=read_matrix_color(f_data,param.used_constr); // Read the constraint matrix for Color/Frequency assignment
	
	fscanf (f_param,"%i",&param.init_file); 
	
	/* any value => an init file is written for future use
	  1=> an init file is read
	*/
	fscanf (f_param,"%s",&param.init_swarm_w);
	f_init_w=fopen(param.init_swarm_w,"w"); 
	printf("\n WARNING:  I will write an init file for future use: %s",param.init_swarm_w);
	printf("\n It may destroy an existent one");
	fprintf(f_init_w,"%i particle_size", param.DD); // An init file is WRITTEN for future use
	
	if (param.init_file>0) 
	{
		fscanf (f_param,"%s",&param.init_swarm_r); // An init file will be READ
		printf("\n Init file %s",param.init_swarm_r);
	}
}

// Volume to check
param.H.volume=1; 
for (d=0;d<param.DD;d++)
{
	param.H.volume=param.H.volume*(param.H.max[d]-param.H.min[d]);
}

return param;
}

// ----------------------------------------------------------------------------- REGRANUL
float	regranul(float x,float granul)
{ // Modify a value according to the granularity of the search space

if (granul<almostzero) return x;  // Pseudo-continuous (depending on the machine)

if (x>=0) return (float)(granul*floor (x/granul+0.5)); //"1/granul" is the minimum distance between two points

return (float)(-granul*floor(-x/granul+0.5));

}

// ----------------------------------------------------------------------------- REHOPE
struct swarm rehope(struct swarm sw,struct param param, struct model model)
{// Try some specific methods to improve positions

struct swarm 	swt;

chance[level]=0;

printf("\n *** REHOPE");

swt=init_swarm(param, model); // Re init the whole swarm
swt.best=best(swt);// ...rank of population PREVIOUS best

if (swt.part[swt.best].prev_best.f>best_result[level].f) // Keep the best position found so far
{
	swt.part[swt.best].pos=best_result[level];
}

return swt;
}

// ----------------------------------------------------------------------------- REMOVE_PARTICLE
struct swarm remove_particle(struct swarm sw,int rank, int Min_size)
{
int				i;
struct swarm 	swt;


if (sw.size<=Min_size) return sw;

//printf("\n remove a particle => %i",sw.size-1);

n_remove=n_remove+1;
swt=sw;

//printf("\n Remove particle %.0f",	swt.part[rank].pos.label);

// Compact
E_p=E_p-swt.part[rank].pos.ep;	
E_k=E_k-swt.part[rank].vel.ek;

if (rank<sw.size-1)
{
	for (i=rank;i<sw.size-1;i++)
	{
		swt.part[i]=swt.part[i+1];
	}
}

swt.size=sw.size-1;	

return swt;
}

// ----------------------------------------------------------------------------- SAVE_ENERGY
void save_energy(FILE *f_energy,struct swarm sw)
{
float	ek,ep;

ek=energy_k_swarm(sw);
ep=energy_p_swarm(sw);
fprintf(f_energy,"%10.0f %i %f %f\n",eval_f_tot[level],sw.size,ep,ek);

}
// ----------------------------------------------------------------------------- SAVE_RESULT
void save_result(FILE *f_run, FILE *f_init_w,struct position best_result,struct param param,int begin, struct model model)
{
int	d;

if (begin>0) goto results;
fprintf(f_run,"\n");
fprintf(f_run,"Data_file ");

fprintf(f_run,"used_constr ");

fprintf(f_run,"Init_swarm_r ");
fprintf(f_run,"Init_swarm_w ");

fprintf(f_run,"funct ");
fprintf(f_run,"Dim. ");

fprintf(f_run,"target ");
fprintf(f_run,"eps ");
fprintf(f_run,"xmin ");
fprintf(f_run,"xmax ");
fprintf(f_run,"granul ");

fprintf(f_run,"alpha_min ");
fprintf(f_run,"alpha_max ");
fprintf(f_run,"b_min ");
fprintf(f_run,"b_max ");

fprintf(f_run,"Max_size ");
fprintf(f_run,"Hood_size ");
fprintf(f_run,"Init_size ");
fprintf(f_run,"Max_swarm ");
fprintf(f_run,"Mean_swarm ");

fprintf(f_run,"printlevel ");

fprintf(f_run,"eval_f ");
fprintf(f_run,"Clock_ticks ");
fprintf(f_run,"Result ");


return;

results:
fprintf(f_run,"\n");
fprintf(f_run,"%s",param.data);

fprintf(f_run," %.1f ",param.used_constr);
fprintf(f_run,"%s",param.init_swarm_r);
fprintf(f_run," %s",param.init_swarm_w);

fprintf(f_run," %2i",param.funct);
if (param.funct==my_function) fprintf(f_run,"/%i",param.mine);
fprintf(f_run," %2i",param.DD);

fprintf(f_run," %4.3f",param.target);
fprintf(f_run," %4.10f",param.eps);
fprintf(f_run," %4.3f",param.H.min[0]);
fprintf(f_run," %4.3f",param.H.max[0]);
fprintf(f_run," %4.3f",param.H.dx[0]);

fprintf(f_run," %4.2f",model.alpha_min);
fprintf(f_run," %4.2f",model.alpha_max);
fprintf(f_run," %4.2f",model.b_min);
fprintf(f_run," %4.2f",model.b_max);

fprintf(f_run," %i",Max_swarm_size);
fprintf(f_run," %i",model.hood_min);
fprintf(f_run," %i",model.hood_max);
fprintf(f_run," %i",model.init_size);
fprintf(f_run," %i",Max_swarm);
fprintf(f_run," %.0f",Mean_swarm);

fprintf(f_run," %2i",param.printlevel);

fprintf(f_run," %6.0f",eval_f_tot[level]);
fprintf(f_run," %ld",clock_tick);
fprintf(f_run," %4.10f",best_result.f);


for (d=0;d<best_result.size;d++)
{
	fprintf(f_run," %4.6f",best_result.x[d]);
}
	
if (param.funct==12) // Special case Frequency Assignment
{
	// WRITE an init file for future use

	fprintf (f_init_w,"\n"); 
	if (best_result.f<param.eps)// Write status
	{
	 	fprintf(f_init_w,"%i ",1);
	}
 	else
	{
		fprintf(f_init_w,"%i ",0);
	}
	 
	for (d=0;d<best_result.size;d++)
	{
		fprintf(f_init_w," %4.3f",best_result.x[d]);
	}
}
}	

// ----------------------------------------------------------------------------- SAVE_SWARM
void save_swarm(FILE *f_swarm,struct swarm sw)
{
int d;
int i;

fprintf(f_swarm,"\n %i %i",sw.size,sw.part[0].pos.size);// Swarm size and dimension

for (i=0;i<sw.size;i++)
{
	fprintf(f_swarm,"\n"); //Current swarm
	for (d=0;d<sw.part[i].pos.size;d++)
	{
		fprintf(f_swarm,"%.0f %3.2f ",sw.part[i].pos.label, sw.part[i].pos.x[d]);
	}
	fprintf (f_swarm," %4.4f",sw.part[i].pos.f);
	
	fprintf(f_swarm,"    "); // Best swarm
	for (d=0;d<sw.part[i].pos.size;d++)
	{
		fprintf(f_swarm,"%3.2f ",sw.part[i].prev_best.x[d]);
	}
	fprintf (f_swarm," %4.4f",sw.part[i].prev_best.f);

}

}

// ----------------------------------------------------------------------------- SAVE_VISU
void save_visu(struct swarm sw,int level,float eps)
{
/* 
  Save some particule informations for post-mortem 2D visualization
See in tot** () functions (like tot_color) how to compute the  values X1 and X2 (global variables) 
See in complete() how to store the values.
  For each particule:
	X1, X2, F: "position" in a given visualization space and function value
	and the same for the best previous position

*/

int 			d;
int				dim;
float			F;
int				g;
struct subswarm hood;
int				i;
float			LABEL;
float			X1,X2;

FILE			*f;

//printf("\n save_visu");

if (level==0) f=f_visu0; else f=f_visu1;

dim=sw.part[0].pos.size;
fprintf(f_visu0,"%i %i\n",sw.size,dim);
	
for (d=0;d<sw.size;d++)
{
	X1=sw.part[d].pos.X1;
	X2=sw.part[d].pos.X2;
	F=sw.part[d].pos.f;	
	LABEL=sw.part[d].pos.label;
		
	//printf("%f %f %f  ",X1,X2,F);
	fprintf(f_visu0,"%.0f ",LABEL);
	fprintf(f_visu0,"%f %f %f  ",X1,X2,F);
		
	X1=sw.part[d].prev_best.X1;
	X2=sw.part[d].prev_best.X2;
	F=sw.part[d].prev_best.f;
	//printf("%f %f %f ",X1,X2,F);
	fprintf(f_visu0,"%f %f %f ",X1,X2,F);

	hood=hood_swarm(sw,d,eps);
	g=hood.rank[hood.best];
	LABEL=sw.part[g].pos.label;
		
	fprintf(f_visu0,"   %.0f",LABEL); // Label of the best neighbour
		
	// Particle position
		
	for (i=0;i<dim;i++)
		fprintf(f_visu0," %f",sw.part[d].pos.x[i]);

	fprintf(f_visu0,"\n");
}
}

// ----------------------------------------------------------------------------- TOT
float tot(struct position pos, struct param param, struct model model)
{
/*
'******************* Total function **************************************** 
' funct: type of function
' input: i (rank of the particle)
' x( , ): position
*/
int				cmax;
int				d;
int 			DD;
int				funct;
int				j;

int				k;
struct model	modelt;
float			product;
double			sum1,sum2;
float 			total;
struct position post={0};
float			x;
float			xid;

static int a[2][25] =
{
   {-32, -16, 0, 16, 32, -32, -16, 0, 16, 32, -32, -16, 0, 16, 32,
      -32, -16, 0, 16, 32, -32, -16, 0, 16, 32  },
   {-32, -32, -32, -32, -32, -16, -16, -16, -16, -16,
      16, 16, 16, 16, 16, 32, 32, 32, 32, 32 }
};

funct=param.funct;

eval_f[level]=eval_f[level]+1;
eval_f_tot[level]=eval_f_tot[level]+1;
if (density_step>0) update_density(pos,param.H,density_step);

if (param.printlevel>2)
	printf("\n tot. eval_f_tot %6.0f",eval_f_tot[level]);

DD=pos.size;

switch (funct)
{
case 1:  //DD-sum
	total = 0;
	for( d=0;d<DD;d++)                       //for each dimension
	{
		total = total + pos.x[ d];               //add DD if i's coordinates
	}	
	
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=pos.f;
		break;
	}
		
	if (DD==2)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}

	x=param.target/(float)DD;
	for (d=0;d<DD;d++)
	{
		post.x[d]=x;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;
	
case 2:  // DD-product. 
	total = 1;
	for( d=0;d<DD;d++)                      
		total = total*pos.x[ d];
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
		
	if (DD==2)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}

	x=(float)pow(param.target,1/(float)DD);
	for (d=0;d<DD;d++)
	{
		post.x[d]=x;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;              
	
	
case 3: // Sphere    
	//      Global min f(x)=0, x(i)=0 
	  
	total = 0;
	for( d=0;d<DD;d++) 
	{                      
		total = total + pos.x[ d]*pos.x[ d];
	}
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
	
	if (DD==2)
	{
		X1=pos.x[0]; // For Leonardo visualization
		X2=pos.x[1];
		break; 
	
	}
	
	X1=(float)sqrt (total); // For Leonardo visualization
	X2=diverg(pos);            
	break;
	
case 4: // Rosenbrock's valley, Banana function
//  Global min f(x)=0, x(i)=1
	total=0;
	
	for (d=0;d<DD-1;d++)
		{ 
		xid=1-pos.x[d]; 
		total=total+xid*xid;
		xid= pos.x[d]*pos.x[d]-pos.x[d+1];
		total=total+100*xid*xid;                
		} 
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
		
	if (DD==2)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}

	for (d=0;d<DD;d++)
	{
		post.x[d]=1;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;
	
	
case 5: // Clerc's f1, Alpine function, min 0 in (0,...,0)
	total=0;
	product=1;
	for( d=0;d<DD;d++) 
		{
		xid=(float)sin(pos.x[d]);
		product=product*xid;
		total=total+(d+1)*pos.x[d]*xid;
		}
	total=(float)((product+1.1)*fabs(total));
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
	
	if (DD==2)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}
	
	x=(float)pow(param.target,1/DD);
	for (d=0;d<DD;d++)
	{
		post.x[d]=x;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;
	
case 6: // Griewank's function. Min =0 at (100,100 ... 100)

	total=0;
	product=1;
	for (d=0;d<DD;d++)
		{
		xid=pos.x[ d]-100;
		total=total+xid*xid;
		product=(float)(product*cos (xid/sqrt(d+1)));
		}
	total=total/4000 -product +1;
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
	
	if (DD==2)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}
	
	x=100;
	for (d=0;d<DD;d++)
	{
		post.x[d]=x;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;
	
	// -------- 2D only ( on [-100,100]^2)
	if (pos.x[1]<0)
		{
		total=(float)(fabs(pos.x[0])+fabs(pos.x[1]+50));
		}
	else
		{
		if (pos.x[0]<0)
			{
			total=(float)(1+fabs(pos.x[0]+50)+fabs(pos.x[1]-50));
			}
		else
			{
			total=(float)(2+fabs(pos.x[0]-50)+fabs(pos.x[1]-50));
			}	
		}
	X1=pos.x[0]; // For Leonardo visualization
	X2=pos.x[1];
	break;

	
case 7:  // Rastrigin. Minimum value 0. Solution (0,0 ...0)
	k=10;

	total=0;
	for (d=0;d<DD;d++)
		{
		xid=pos.x[ d];
		total=(float)(total+xid*xid - k*cos (2*3.14159265*xid)+10);

		//total=(float)(total+xid*xid - k*cos (two_pi*xid));
		}
	//total=total+(float)(DD*k);
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
	
	if (DD==2)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}
	
	x=0;
	for (d=0;d<DD;d++)
	{
		post.x[d]=x;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;

case 8:  // Fifty-fifty problem
	total=tot_fifty_fifty(pos);
	X1=total;
	X2=diverg(pos);
	break;
	
case 9:  // Ackley
	sum1=0;
	sum2=0;
	x=(float)DD;
	for (d=0;d<DD;d++)
		{
		xid=pos.x[ d];
		sum1=sum1+xid*xid;
		sum2=sum2+cos(two_pi*xid);
		}
	total=(float)(-20*exp(-0.2*sqrt(sum1/x))-exp(sum2/x)+20+E);
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
	if (DD==2)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}

	x=0;
	for (d=0;d<DD;d++)
	{
		post.x[d]=x;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;	
	
	
case 10: //Foxholes 2D
	total=0;
	for (j=0; j<25; j++)
   		{
		sum1=0;
      	for (d=0; d<2; d++)
      		{
         	sum1 =sum1+ pow (pos.x[d] - a[d][j],6);
      		}
      	total=(float)(total+1/(j+1+sum1));
   		}
	total=(float)(1/(0.002+total));
	
	if (DD==1)
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
	if (DD<=2)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}
	
	x=0;
	for (d=0;d<DD;d++)
	{
		post.x[d]=x;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;	
	
case 11: // Apple trees
	total=apple_trees(pos,param.printlevel);
	post= homogen_to_carte(pos); // For Leonardo visualization
	X1=total;
	X2=diverg(post);
	break;
	
case 12: // Coloring problem/Frequency assignement
		// M is a global variable (matrix of the graph)
	cmax=(int)param.max_fr;
	total=tot_color(pos,M,cmax,param.save);
	// Note: X1 and X2 are calculated in tot_color
	break;	

case 13: // PSO for PSO

/*
Adaptive PSO has n parameters ( see pos_to_model() )

A set of values for these parameters is a "position" of a  (meta) particle.
Let P be a given problem. 
To solve it with a given "position", PSO needs T evaluations: 
this is the objective function value for this "position".
We are trying to minimize T, taking into account some constraints:
alpha_min<=alpha_max
b_min<=b_max

 
*/
	level=level+1;
	recursive=(int)MIN(2,(float)(recursive+1));
	
	eval_f[level]=0;
	eval_f_tot[level]=0;

	modelt=pos_to_model(pos,model);
	total=PSO(recursive,modelt); /* The very first time, recursive=1, to read next lines (second part)
	                              of the parameter file. After that, recursive =2
								*/
	if (param.printlevel>1) 
	{
		display_position (pos);
		printf("/true eval: %f",total);
	}							
	level=level-1;
	break;	

/*	
case 14: // For projection of a coloring on the space of admissible colorings
		// M and pos0 are global variables
	cmax=param.max_fr;
	total= tot_color_proj(pos,M,param.printlevel,cmax,pos0);
	break;
*/

case 15: // El Farol
/* 
1 dimension = 1 Irishman
Just two values/dim.: 0 <=> stay at home
                      1 <=> go to the pub

Note: the current swarm sw is a global variable
*/
	sum1=60; // Maximum "tolerance": maximum number of people each Irishman does accept in the pub
	total=0;
	for (d=0;d<DD;d++)
	{
		total=total+pos.x[d];
	}

	total=(float)(total*(1-total/sum1));
	break;

	


case my_function: // MyFunction
	total=MyFunction(pos,param, model);
	
	if (DD==1)// For Leonardo visualization
	{
		X1=pos.x[0]; 
		X2=(float)fabs(total-param.target);
		break;
	}
	
	if (DD<=2)
	{
		X1=pos.x[0]; 
		X2=pos.x[1];
		break; 
	}
	
	x=0;
	for (d=0;d<DD;d++)
	{
		post.x[d]=x;
	}
	X1=distance(pos,post);
	X2=diverg(pos);
	break;
	
default:
	printf("\n ERROR, unknown objective function %i",funct);
	total=(float)NA;
	
}

return total;
}

// ----------------------------------------------------------------------------- TOT_FIFTY_FIFTY
float tot_fifty_fifty(struct position pos) 

// Sum of first numbers = sum of last numbers

{
int		a;
int		d;
int		d2;
float	tax_twice;
float	tot_f;
float	xa,xb;

a=(int)(0.5*(pos.size+1));

xa=0;
for (d=0;d<a;d++)
	{
	xa=xa+pos.x[d];
	}
	
xb=0;
for (d=a;d<pos.size;d++)
	{
	xb=xb+pos.x[d];
	}
	
tot_f=(float)fabs(xa-xb);

return tot_f;

// Old method
tax_twice=(float)(0.5*pos.size*(pos.size+1));
for (d=0;d<pos.size-1;d++) // Penalties if twice the same number
	{
	xa=pos.x[d];
	for (d2=d+1;d2<pos.size;d2++)
		{
		xb=pos.x[d2];
		if (xa==xb)
			tot_f=tot_f+tax_twice;
		}
	}

return tot_f;

}

// ----------------------------------------------------------------------------- UPDATE_DENSITY
void update_density(struct position pos,struct search_space H,int step)
{
// Note: density[step][Max_DD] is a global variable


int		d;
int		i;
float	dx;
float	xd;

for (d=0;d<pos.size;d++)
{
	xd=pos.x[d];
	dx=(H.max[d]-H.min[d])/(float)step;
	
	for (i=1;i<=step;i++)
	{
		if (xd>=H.min[d]+dx*(step-i))
		{
			density[step-i][d]=density[step-i][d]+1;
			goto next_d;
		}
	}
next_d:;
}
}


// ----------------------------------------------------------------------------- VEL_GRANUL
struct velocity	vel_granul(struct velocity vel,struct search_space H)
{
int				d;
struct velocity v;

v=vel;
for (d=0;d<vel.size;d++)
{
	v.v[d]=regranul(v.v[d],H.dx[d]);	
}

return v;
}


// ----------------------------------------------------------------------------- WORST
int worst(struct swarm sw)// Find the "previous worst" particle in the swarm
{
int i;
int	sw_worst;

sw_worst=0; // First, arbitrary value for the rank 

for( i = 1;i<sw.size;i++)  // Look for a worse one
{
	if (sw.part[sw_worst].prev_best.f<sw.part[i].prev_best.f)
		sw_worst=i;
}
return sw_worst;
}

